%    plot_Robot
%
%    Graphic function for 3D manipulators plot.
%
%    Usage: plot_Robot(P,Z,EE,joints,q,varargin)
%
%    Inputs:
%    P = matrix whose i-th column is the axis origin for the i-th joint
%    Z = matrix whose i-th column represent the i-th joint axis. 
%    joints = vector containing the prismatic joints configuration
%             (0 for rotational, 1 for prismatic joints).
%    q = joint variables vector, represents the configuration in which will
%    be plotted the manipulator.(if a single 0 is given, means that all the
%    joints configuration is 0).
%    varargin = variable input arguments, representing each chain of the
%    manipulator.

function plot_Humanoid( P_0, Z_0, g_st0, q_actual, Cc, vkc_dim, varargin)
%
figure

if isempty(findobj(gcf,'type','axes'))
    ax = axes;
    grid on;
    axis equal;
    camlight left;
    set(ax,'XLimMode','auto');
    set(ax,'YLimMode','auto');
    set(ax,'ZLimMode','auto');
    xlabel('X');ylabel('Y');zlabel('Z');
else
    ax = gca;
end

% %%user-defined geometric parameters
scaling_c = .1;
scaling_r = .002;
scaling_s = .02;
%     scaling_c = 1;
%     scaling_r = 1;
%     scaling_s = 1;
params.links_radius = 0.02 ; %scaling_r*1;
params.joints_radius = 1.5; %0.8 ;%scaling_c*1;
params.joints_height = 3 ;%scaling_c*4;
params.view = [0,90];
params.scaling = 1/scaling_s;
params.alpha = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(size(varargin,1)~=0)
    joint_type = varargin{1} ;
else
    joint_type = zeros(1,size(Z_0,2)) ;
end
qd = size(P_0,2);
% if a single 0 is given, means that all the joints configuration is 0
if( size(joint_type,1) == 1 && size(joint_type,2) == 1 && ((joint_type == 1) || (joint_type == 0)))
    joint_type = joint_type*ones(qd);
else
    if( size(joint_type,1) == qd || size(joint_type,2) == qd )
    else
        error('wrong value specyfied for joint_type: expected a single value or a 1 x num_joints vector');
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
P = P_0  ; % ( :,vkc_dim:end) ;
Z = Z_0 ; %(:,vkc_dim:end) ;
q = q_actual  ; %(vkc_dim:end) ;
%joint_type = joint_type (vkc_dim:end);
Cc_robot = Cc ; %(:,vkc_dim:end)  ;
num_chains = size(Cc_robot,1) ;

EE = zeros(3,size(g_st0,3)) ;
for i = 1 : size(g_st0,3)
   EE(:,i)= g_st0(1:3,4,i) ;
end

for i = 1 : num_chains
    chain_i = find(Cc_robot(i,:)) ; %varargin{i};
    % base transformations for links and joints
    g0 = base2frame_serialchain( P , EE(:,i) , chain_i ) ;
    g0JOINTS = base2frame_serialchainJOINTS( P , Z , chain_i );

    P_ = [];
    Z_ = [];
    q_ = [];
    joint_type_ = [] ;
    for k = 1 : size(chain_i,2)   % for each joint belonging to the serial chain

        P_ = [  P_   P(:,chain_i(k))  ] ;
        Z_ = [  Z_   Z(:,chain_i(k))  ] ;
        q_ = [  q_   q(chain_i(k))    ] ;
        joint_type_ = [joint_type_   joint_type(chain_i(k))] ;
        % Calls the forward kinematics function. Each homog(:,:,i)
        % represents the homogeneous transformation from base to (i+1)th joint or link.

        homog_base2frameLINKS(:,:,k) = fw_kin_serial(P_,Z_,g0(:,:,k),q_,joint_type_) ;   
        homog_base2frameJOINTS(:,:,k) = fw_kin_serial(P_,Z_,g0JOINTS(:,:,k),q_,joint_type_) ;
%         if(k == 1)
%             homog_base2frameJOINTS(:,:,1) = homogeneous( new_alignZ(Z(:,chain_i(1)) ),P(:,chain_i(1)) ) ;
%         else
%             homog_base2frameJOINTS(:,:,k) = fw_kin_serial(P_,Z_,g0JOINTS(:,:,k),q_,joint_type_) ;
%         end
    end
    size_current_chain = size(homog_base2frameJOINTS,3) ;
    homog_base2frameLINKS_robot  = homog_base2frameLINKS(:,:,vkc_dim:size_current_chain) ;
    homog_base2frameJOINTS_robot = homog_base2frameJOINTS(:,:,vkc_dim:size_current_chain) ;
    joint_type_robot = joint_type_(vkc_dim:size_current_chain) ;
    vkc_dim_ = vkc_dim-1 ;
    if( vkc_dim_<1)
    base = P(:,chain_i(vkc_dim)) ;
    else
        base = homog_base2frameLINKS(1:3,4,vkc_dim_) ;  % P(:,chain_i(vkc_dim))
    end
    %plot_core(homog_base2frameLINKS, homog_base2frameJOINTS, P(:,chain_i(1)), joint_type_, params);
    plot_core(homog_base2frameLINKS_robot, homog_base2frameJOINTS_robot,base, joint_type_robot, params);
    %hold on;
end
 
view(params.view);
camlight left;

end

function plot_core(links_fkine, joints_fkine, base, joint_type, params)

    links_radius = params.links_radius;
    joints_radius = params.joints_radius;
    joints_height = params.joints_height;
    scaling = params.scaling;
    
    linknum = size(links_fkine,3);
   
    LinksHandlesVect = zeros(linknum,1);
    JointsHandlesVect = zeros(linknum,1);
    terminal = 0;
    
    for k = 1 : linknum
        if(k == 1)
            length_i = norm(links_fkine(1:3,4,k)-base);
            rotrasl_link = homogeneous(links_fkine(1:3,1:3,k),(links_fkine(1:3,4,k)-base)); %base 
        else
            length_i = norm(links_fkine(1:3,4,k)-links_fkine(1:3,4,k-1));
            rotrasl_link = homogeneous(links_fkine(1:3,1:3,k),links_fkine(1:3,4,k-1));
        end

        if(k == linknum)
            terminal = 1;
        end
                    % verde scuro [.1,.255,.3]; [.1,.4,.3] %%% arancio [(255/255),(120/255),(.0/255)]
        rotrasl_joint = joints_fkine(:,:,k);
        LinksHandlesVect(k)= srv_makelink(links_radius,[.1,.45,.3],length_i,terminal,joint_type(k));%'b',length_i,terminal,joints(k));
        set(LinksHandlesVect(k),'Matrix',homogeneous(rot( rotrasl_link),transl(rotrasl_joint) )   ) ; %rotrasl_link);
        JointsHandlesVect(k) = srv_makejoint(joints_radius,joints_height,[1,.2,0],scaling);%'r',scaling);
        set(JointsHandlesVect(k),'Matrix',rotrasl_joint);
                
        alpha(params.alpha);        
    end
end

function h_obj = srv_makelink(radius, link_color, length, terminal, prismatic)

nf = 60;  % facets number
axis_handle = gca;
h_obj = hgtransform('Parent',axis_handle);
if(length)
if(prismatic)
    
    [x1,y1,z1]=cylinder(radius,nf);
    h1 = surface(x1,y1,z1*length/2,'FaceColor',link_color);
    set(h1,'EdgeColor','none');
    set(h1,'Parent',h_obj);
   
    [x2,y2,z2] = cylinder(radius*0.8,6);
    if(terminal)
        h2 = surface(x2,y2,z2*(length/2 - radius),'FaceColor',link_color);
    else
        h2 = surface(x2,y2,z2*length/2,'FaceColor',link_color);
    end
    
    set(h2,'EdgeColor','none');
    hs2 = hgtransform('Parent',axis_handle);
    T = makehgtform('translate',[0,0,length/2]);
    set(h2,'Parent',hs2);
    set(hs2,'Matrix',T);
    set(hs2, 'Parent', h_obj);
   
    [x3,y3,z3]=cylinder([radius 0]);
    h3 = surface(x3,y3,z3*0.01,'FaceColor',link_color);
    set(h3,'EdgeColor','none');
    hs3 = hgtransform('Parent',axis_handle);
    T1 = makehgtform('translate',[0,0,length/2]);
    set(h3,'Parent',hs3);
    set(hs3,'Matrix',T1);
    set(hs3, 'Parent', h_obj);

else

    [x1,y1,z1]=cylinder(radius,nf);
    if(terminal)
        h1 = surface(x1,y1,z1*(length - radius),'FaceColor',link_color);
    else
        h1 = surface(x1,y1,z1*length,'FaceColor',link_color);
    end
        
    set(h1,'EdgeColor','none');
    set(h1,'Parent',h_obj);  
    
end

if(terminal)
    [x3,y3,z3]=sphere(nf);
    z3(z3<0)=0;
    h3 = surface(radius*x3,radius*y3,radius*z3,'FaceColor',link_color);
    set(h3,'EdgeColor','none');
    hs=hgtransform('Parent',axis_handle);
    T=makehgtform('translate',[0,0,length-radius]);
    set(h3, 'Parent', hs);
    set(hs,'Matrix',T);
    set(hs,'Parent',h_obj);
end
end

end

function h_obj = srv_makejoint(radius, height, joint_col, scaling)
 
nf = 60; % number of facets

X = circle([0 0 0],(radius+.25)/scaling,'n',100);

axis_handle = gca;
h_obj = hgtransform('Parent',axis_handle);

% joint body
[x1,y1,z1]=cylinder((radius+(0.25))/scaling,nf);
h1 = surface(x1,y1,(z1*height)/scaling,'FaceColor',joint_col);
set(h1,'EdgeColor','none');
hs1 = hgtransform('Parent',axis_handle);
T = makehgtform('translate',[0,0,-(height/2)/scaling]);
set(h1,'Parent',hs1);
set(hs1,'Matrix',T);
set(hs1,'Parent',h_obj);

% joint lateral facets
h2 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor',joint_col);
set(h2,'EdgeColor','none');
hs2 = hgtransform('Parent',axis_handle);
T = makehgtform('zrotate',pi/2,'translate',[0,0,(height/2)/scaling]);
set(h2,'Parent',hs2);
set(hs2,'Matrix',T);
set(hs2,'Parent', h_obj); 


h3 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor',joint_col);
set(h3,'EdgeColor','none');
hs3 = hgtransform('Parent',axis_handle);
T = makehgtform('zrotate',pi/2,'translate',[0,0,-(height/2)/scaling]);
set(h3,'Parent',hs3);
set(hs3,'Matrix',T);
set(hs3,'Parent', h_obj); 


end