%    plot_Object
%
%    The function plots the manipulated object. 
%
%    Usage: plot_Object(Rotrasl,radius,height,type)
%
%    Inputs:
%    Rotrasl = homogeneous transformation representing traslation and
%    rotation of the manipulated object from the base frame
%    radius = radius of the sphere/cylinder
%    height = (only for cylinder) represents the height of the cylinder
%    type = specify the geometrig figure of the manipulated object

function plot_Object(Rotrasl,radius,height,type)


if isempty(findobj(gcf,'type','axes'))
    ax = axes;
    grid on;
    axis equal;
    %camlight left;
    set(ax,'XLimMode','auto');
    set(ax,'YLimMode','auto');
    set(ax,'ZLimMode','auto');
else
    ax = gca;
end

switch(type)

    case 'Sphere'
        
        [x1,y1,z1]=sphere(60);
        h = surface(x1*radius,y1*radius,z1*radius,'FaceColor','k');
        set(h,'EdgeColor','none');
        hs = hgtransform('Parent',ax);
        set(h,'Parent',hs);
        set(hs,'Matrix',Rotrasl);
    


    case 'Cylinder'

        X = circle([0 0 0],radius,'n',100);

        h = hgtransform('Parent',ax);

        % cylinder body
        [x1,y1,z1]=cylinder(radius,60);
        h1 = surface(x1,y1,(z1*height),'FaceColor','r');
        set(h1,'EdgeColor','none');
        hs1 = hgtransform('Parent',h);
        T1 = makehgtform('translate',[0,0,-(height)/2]);
        set(h1,'Parent',hs1);
        set(hs1,'Matrix',T1);
        set(hs1,'Parent',h);

        % cylinder base
        h2 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor','r');
        set(h2,'EdgeColor','none');
        hs2 = hgtransform('Parent',h);
        T2 = makehgtform('translate',[0,0,-(height)/2]);
        set(h2,'Parent',hs2);
        set(hs2,'Matrix',T2);
        set(hs2,'Parent',h);

        % cylinder top
        h3 = patch(X(1,:)',X(2,:)',X(3,:)','FaceColor','r');
        set(h3,'EdgeColor','none');
        hs3 = hgtransform('Parent',h);
        T3 = makehgtform('translate',[0,0,(height)/2]);
        set(h3,'Parent',hs3);
        set(hs3,'Matrix',T3);
        set(hs3,'Parent',h);
        
        set(h,'Matrix',Rotrasl);

    otherwise
        error('MANIPULATEDOBJECT: no plot metod defined for ''%s''', mo.name);
            
end



end


