%   D_Jt_f_serial
%
%   The function calculates the derivative of J^T*f with respect to q
%   for a serial kinematic chain
%    
%   [ Q_ci ] = D_Jt_f_serial( P, Z, EE_ci_0, q, B_ci, f_ci, (opt.)joint_type )
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   EE_ci_0 \in R^{4 X 4} = homogeneous transformation describing the pose
%   of the contact frame w.r.t {A} in the -zero- configuration
%   q \in R^{1 X (numb. of joints) } = actual configuration of the robot
%   B_ci \in R^{6 X (contact force components) } =  transmittable wrench matrix, its transpose is 
%    a Selection matrix, selecting only the components of the contact twist violating the
%   kinematic contact constraint
%   f_ci \in R^{contact force components } = actual force exerted on the contact 
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   Q_ci \in R^{(numb. of joints) X (numb. of joints) } = derivative of J^T*f with respect to q
%
function [ Q_ci ] = D_Jt_f_serial( P, Z, EE_ci_0, q, B_ci, f_ci, varargin  )
%
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 7) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
J_spa_ci = Spatial_Jacobian_serial(P, Z, q, joint_type ) ;
g_aci_0 = EE_ci_0;
B_ci_T = B_ci';
%
Q_ci = zeros(size(J_spa_ci,2),size(J_spa_ci,2));
for i= 1:size(J_spa_ci,2)
Q_ci(:,i) = (( D_Jacob_spa_i(J_spa_ci,i))'*(ad2(ihomogeneous(g_aci_0)))'...
    * B_ci_T'*f_ci ...
    +...
    J_spa_ci'* ((ad2(ihomogeneous(g_aci_0))*ad_lie(-J_spa_ci(:,i) ) )' )...
    *B_ci_T'*f_ci) ;
end
end
