% D_Jt_mg
% 
%   The function computes the derivative of J_mg^T*(-mg) with respect to q
%   for a serial kinematic chain.
%   NOTE: this function is a special version of D_Jt_f_serial, built
%   considering the peculiarities of the gravitational terms w.r.t. the
%   usual contact forces, i.e. the force-application frame never rotate
%   the term "-mg" is the gravitational force applyied by the robot
%    
%   [ Q_g ] = D_Jt_mg( P_g, Z_g, EE_g, q_g, B_g, f_g, (opt.)joint_type )
%
%   Inputs:
%   P_g \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z_g \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   EE_g \in R^{4 X 4} = homogeneous transformation describing the pose
%        of the contact frame w.r.t {A} in the -zero- configuration 
%        (rotational part is neglected => f_g is expressed in absolute coordinates)
%   q_g \in R^{1 X (numb. of joints) } = actual configuration of the robot
%   B_g \in R^{6 X (contact force components) } = transmittable wrench matrix, its transpose is 
%        a Selection matrix, selecting only the components of interest (useful for planar problems)
%   f_g \in R^{contact force components } = gravitational force exerted by
%   the robot ( f_g = -mg )
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   Q_g \in R^{(numb. of joints) X (numb. of joints) } = derivative of J^T*f with respect to q
%
function [ Q_g ] = D_Jt_mg(P, Z, g_m_0, q, Cc, m, B_g, f_g, varargin ) 
%( P_g, Z_g, EE_g, q_g, B_g, f_g , varargin  )
%
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 9) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
h = sqrt(eps) ;
% 
% M = sum(m) ;
% f_g = 10*M ;

J_com_0 = J_CoM(P, Z, g_m_0, q, Cc, m,joint_type) ;
Q_g_0 = J_com_0'*B_g*f_g ;

%
Q_g = zeros(size(J_com_0,2)) ;
%
for i = 1 : size(J_com_0,2)
    q_g_h = q ;
    q_g_h(i) = q_g_h(i) + h ;   
    J_com_h = J_CoM(P, Z, g_m_0, q_g_h , Cc, m,joint_type) ;
    Q_g(:,i) = (J_com_h'*B_g*f_g - Q_g_0)/h ;
end
%
end










