%   Body_Jacobian_serial
%
%   The function calculates the body Jacobian for a serial kinematic chain
%   in the given configuration
%    
%   Syntax: Body_J_serial = Body_Jacobian_serial(P, Z, g_st0, q, (optional)joint_type ) 
%
%   Inputs:
%   P \in R^{3 X (numb. of joints) } = Collecnting points through the
%        joint axis in the -zero- configuration of the robot
%   Z \in R^{3 X (numb. of joints) } = Collecnting joint axis vectors
%        in the -zero- configuration of the robot
%   g_st0 \in R^{4 X 4 } = homogenous trasformation matrix from base to end-effector frame 
%   q \in R^{1 X (numb. of joints) } = 
%           Joint configuration in which will be evaluated the forward kinematics 
%   joint_type \in R^{1 X (numb. of joints) } =  OPTIONAL INPUT Vector
%           the i-th element of the vector is 1 (true) if the i-th joint is PRISMATIC
%           the i-th element of the vector is 0 (false) if the i-th joint is REVOLUTE
%   if joint_type is empty, all the joints are considered revolute, if it
%   is a scalar, all the joints are of the same type
%
%   Outputs:
%   Body_J_serial \in R^{6 X (numb. of joints) } = Body Jacobian of the kinematic chain
%
function Body_J_serial = Body_Jacobian_serial(P, Z, g_st0, q, varargin ) %
%
joint_type = zeros(1,size(Z,2)) ;
if(nargin >= 4) %
    vett = varargin{1};
    if (length(vett)==1)
        joint_type = vett*ones(1,size(Z,2)) ;
    else
        for i = 1 : length(vett)
            joint_type(i) = vett(i);
        end
    end
end
%
Spatial_J = Spatial_Jacobian_serial(P, Z, q, joint_type ) ;
T_ab = fw_kin_serial(P,Z,g_st0,q,joint_type) ;
%
Body_J_serial = ad2(ihomogeneous(T_ab))*Spatial_J ;
%
end




