% FLMM_compact_elastic
%
% Provide a compact form  (elastic robot, no COM displacement, etc...  
% WITHOUT external wrench on the robot)
% of the Fundamental Loco-Manipulation Matrix
% for an elastic humanoid robots
%
%   Syntax: [ cFLMM ] = cFLMM_compact_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c , K_q )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%   K_q \in R^{(numb. of joints) X (numb. of joints)} = 
%               Joint stiffness matrix (usually it is diagonal) %
%
%   Outputs:
%   cFLMM is the canonical form "Fundamental Loco-Manipulation Matrix", WITHOUT
%   external wrench, for an elastic robot
%
function [ cFLMM ] = cFLMM_compact_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q )
%
FLMM = FLMM_compact_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c , K_q ) ;
cFLMM  = FLMM(:,1:size(FLMM,1))\FLMM ;
%
end

