% E_rigid
%
% Provide a base of the controllable contact forces,
% such that d_fc = E y
% for rigid humanoid robot
%
%   Syntax:  [ E ] = E_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
% Outputs:
% E is a base for the controllable contact forces,
% such that d_fc = E y, where y is a suitable coefficient vector  
%
function [ E ] = E_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, varargin )
%
if(size(varargin,1)~=0)
    tol = varargin{1} ;
else
    tol = 1E-2;
end
%
R_f_temp = Rf_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, tol ) ;
%
temp = R_f_temp./(10.^(floor(log10(abs(R_f_temp))))) ;
temp_2 = nearest(temp.*.2)./.2 ; % changing the granularity for the nearest
Rf = temp_2.*(10.^(floor(log10(abs(R_f_temp))))) ;
E_temp = orth(Rf) ;
E = rref(E_temp')' ;
% Rf = fix(R_f_temp./(10.^(floor(log10(abs(R_f_temp)))))) .*(10.^(floor(log10(abs(R_f_temp))))) ;
% E_temp = orth(Rf) ;
% E = rref(E_temp')' ;
% 
end