% FLMM_i_rigid
%
% Provide the block relative to the "independent variables" of the
% Fundamental Loco-Manipulation Matrix
%
%   Syntax:  [ FLMM ] = FLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
%   Outputs:
%   FLMM_i_rigid is block relative to the "independent variables" of the "Fundamental Loco-Manipulation Matrix"
%
function [ F_i ] = FLMM_i_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
FLMM = FLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c ) ;
F_i = FLMM(:, size(FLMM,1)+1:end)  ;
%
end