% Rtau_rigid
%
% Provide the map between joint displacement and floating base displacement (VKC joint displacemnt),
% such that d_u = -Ru d_q
% for rigid humanoid robot
%
%   Syntax:  [ Ru ] = Ru_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
%   Outputs:
%   Rtau_rigid is the map between joint displacement and floating base displacement (VKC joint displacemnt),
% such that d_u = -Ru d_q
% for rigid humanoid robot
%
function [ Ru ] = Ru_rigid( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, varargin )
%
if(size(varargin,1)~=0)
    tol = varargin{1} ;
else
    tol = 1E-2;
end
%
s_w = size(S_c,1) ; 
s_q  = size(J_c,2) ; 
%
cFLMM  = cFLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c );
Ru_dirty = cFLMM(end-s_w+1:end, end-s_q+1:end) ;
% Ru_dirty_temp = Ru_dirty ;
% Ru_dirty_temp(~Ru_dirty_temp)=inf ;
% minimum = min(min(abs(Ru_dirty_temp))) ; 
Ru = Ru_dirty ; % round(Ru_dirty./(minimum*tol))*minimum*tol ;
% Ru = fix(Ru_temp./(10.^(floor(log10(abs(Ru_temp)))))) .*(10.^(floor(log10(abs(Ru_temp))))) ;
end


