% cFLMM_compact_rigid
%
% Provide the canonical form of the
% Fundamental Loco-Manipulation Matrix, in compact form, WITHOUT external wrench
%
%   Syntax:  [ cFLMM_rigid ] = cFLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
%   Outputs:
%   cFLMM_rigid is the canonical form of the "Fundamental Loco-Manipulation
%   Matrix", in compact form, WITHOUT external wrench
%
function [ cFLMM_rigid ] = cFLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
FLMM = FLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c ) ;
cFLMM_rigid  = FLMM(:,1:size(FLMM,1))\FLMM ;
%
end

