% From a generic displacement vector u, returns the correspondent
% homogeneous trasformation matrix, checking in the configuration if the
% problem is 2D, and so which element of u is a displacement end from which
% axis it belongs.

function out = base_to_object_transformation( u )

try
    srv_axis = evalin('base','srv_axis');
catch
    srv_axis = 'xyz';
end

switch srv_axis
    
    case 'xy'
        if( (size(u,1) == 3) || (size(u,2) == 3) )
            out = homogeneous(rotz(u(3)), [u(1) u(2) 0]);
        else
            out = homogeneous(rotz(u(6)), [u(1) u(2) 0]);
        end
        
    case 'xz'
        if( (size(u,1) == 3) || (size(u,2) == 3) )
            out = homogeneous(roty(u(3)), [u(1) 0 u(2)]);
        else
            out = homogeneous(roty(u(5)), [u(1) 0 u(3)]);
        end
           
    case 'yz'
        if( (size(u,1) == 3) || (size(u,2) == 3) )
            out = homogeneous(rotx(u(3)), [0 u(1) u(2)]);
        else
            out = homogeneous(rotx(u(4)), [0 u(2) u(3)]);
        end
              
    case 'xyz'
        out = homogeneous(rotx(u(4))*roty(u(5))*rotz(u(6)), [u(1) u(2) u(3)]);
end

end

