% Rq_elastic
%
% Provide the map between joint displacement and joint torque variation,
% such that d_q = -Rq d_q_r
% for elastic humanoid robot
%
%   Syntax:  [ Rtau ] = Rq_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%   K_q \in R^{(numb. of joints) X (numb. of joints)} = 
%               Joint stiffness matrix (usually it is diagonal) %
%
%   Outputs:
%   Rq_elastic is the map between joint displacement and joint torque variation,
% such that d_q = -Rtau d_q_r
% for elastic humanoid robot
%
function [ Rq ] = Rq_elastic( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q,  varargin )
%
if(size(varargin,1)~=0)
    tol = varargin{1} ;
else
    tol = 1E0;
end
%
s_q  = size(J_c,2) ; 
%
cFLMM  = cFLMM_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c, K_q );
R_q_dirty = cFLMM(end-s_q+1:end, end-s_q+1:end) ;
% R_q_dirty_temp = R_q_dirty ;
% R_q_dirty_temp(~R_q_dirty_temp)=inf ;
% minimum = min(min(abs(R_q_dirty_temp))) ; 
Rq = R_q_dirty; %round(R_q_dirty./(minimum*tol))*minimum*tol ;
%
end