% cFLMM_elastic
%
% Provide the canonical form of the
% Fundamental Loco-Manipulation Matrix for an elastic robot
%
%   Syntax:  [ cFLMM ] = cFLMM_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c , K_q )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%   K_q \in R^{(numb. of joints) X (numb. of joints)} = 
%               Joint stiffness matrix (usually it is diagonal) 
%
%   Outputs:
%   cFLMM is the canonical form of the "Fundamental Loco-Manipulation
%   Matrix" for an elastic robot
%
function [ cFLMM ] = cFLMM_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c , K_q )
%
FLMM = FLMM_elastic(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c , K_q ) ;
cFLMM  = FLMM(:,1:size(FLMM,1))\FLMM ;
%
end