% FLMM_rigid
%
% Provide a compact form  (rigid, no COM displacement, etc...  
% WITHOUT external wrench on the robo)
% of the Fundamental Loco-Manipulation Matrix
% for rigig humanoid robots
%
%   Syntax:  [ FLMM_compact ] = FLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
%   Inputs:
%   J_c \in R^{(numb. of contact force elements) X (numb. of joints) } = 
%               is the Humanoid Jacobian, relative to the cotnact points
%   S_c \in R^{ (numb. of joints of the VKC) X (numb. of contact force elements) } = 
%               is the Stance Matrix, relative to the cotnact points
%   Q_j \in R^{(numb. of joints) X (numb. of joints)  } = derivatice term = (d J^t*F_c)/(d q)
%   Q_s \in R^{(numb. of joints of the VKC) X (numb. of joints) } =  derivatice term = (d S*F_c)/(d q)
%   U_j \in R^{(numb. of joints) X (numb. of joints of the VKC) } = derivatice term = (d J^t*F_c)/(d u)
%   U_s \in R^{(numb. of joints of the VKC) X (numb. of joints of the VKC) } = derivatice term = (d S*F_c)/(d q)
%   K_c \in R^{(numb. of contact force elements) X (numb. of contact force elements) } = 
%               Contact stiffness matrix (usually it is diagonal)
%
%   Outputs:
%   FLMM_compact is the "Fundamental Loco-Manipulation Matrix", if no
%   external wrench exists
%
function [ FLMM_compact ] = FLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  K_c )
%
s_fc  = size(J_c,1) ; 
s_w = size(S_c,1) ; 
s_q  = size(J_c,2) ; 
%
FLMM_compact = [ ...
    eye(s_fc),   zeros(s_fc,s_q),     -K_c*S_c',      -K_c*J_c ;...
    -J_c',        eye(s_q),             -U_j,          -Q_j;...
    -S_c,       zeros(s_w,s_q),        -U_s,          -Q_s   ];

end

