% V_tot
%
%   Computes the fucntion V (to be minimized) for the current contact force distribution.
%   Friction cone, minimum and maximum normal force are considered
%
%   Syntax: [ V_final ] = V_tot( f_c, normals, mu, f_min, f_max , cf_dim )
%
%   Inputs:
%   f_c     is a vector collecting all the current contact forces
%       Only linear forces are considered (f_ci is \in R^{3}, or \in R^{2} for planar problems). 
%   normals is a vector collecting di directions of all the normals a the contact points
%       (n_i is \in R^{3}, or \in R^{2} for planar problems). 
%   mu    is the vector collecting all the friction coefficient at the contacts
%   f_min is the vector collecting all the minimum normal forces admitted at the contacts
%   f_max is the vector collecting all the maximum normal forces admitted at the contacts
%
% Outputs:
% V_tot is term of the fucntion V relative to the i-th contact
%
function [ V_final ] = V_tot( f_c, normals, mu, f_min, f_max , cf_dim )
%
start_i = zeros(size(cf_dim)); % index of start of the fc_i vector
end_i = zeros(size(cf_dim)); % index of end of the fc_i vector
%
start_i(1) = 1 ;
for j = 2:length(cf_dim)
    start_i(j) = start_i(j-1)+cf_dim(j-1); 
end
for j = 1:length(cf_dim)
    end_i(j) = start_i(j) + cf_dim(j)-1 ;
end
V = 0;
for i = 1: length(cf_dim)
   V =V + V_i( f_c(start_i(i):end_i(i)), ...
                     normals(start_i(i):end_i(i)),...
                     mu(i), f_min(i), f_max(i) ) ;
end
%
V_final = V ;
%
end
%
