%
% This is an example explointing the funtionalities of the toolbox
%
clear all 
close all
clc
format short
%
% Some useful parameters of the problem
% Weight of the Robot
mg_0 = - 500 ;  % [kg]
% Inclination of the surface
alpha_deg_0 = 20 % (deg)
%
% Fricition Coefficients on the contacts
mu_1 =  1
mu_2 =  1
mu_vect = [mu_1, mu_2] ;
%
f_min = 0 ;             % [N] minimum normal contact force allowable
f_max_leg = abs(mg_0) ; % [N] maximum normal contact force allowable 
%
f_min_vect = [f_min, f_min];
f_max_vect = [f_max_leg, f_max_leg];
%
n_c1 = [0 -1]' ;
n_c2 = [cos( deg2rad(alpha_deg_0)-pi/2) ; sin(deg2rad(alpha_deg_0)-pi/2)  ]  ;
normals_vect = [n_c1; n_c2] ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
alpha = deg2rad(alpha_deg_0) ; % (rad)
%
% Link Lengths 
L1 = 0.5  ; % [m]
L2 = 0.5  ; % [m]
L3 = 0.5  ; % [m]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parametrized initial robot condiguration
beta_deg_0 = 60 ;
beta_0 = deg2rad(beta_deg_0) ;
D = sqrt( L2^2+ L3^2 -2*L2*L3*cos(beta_0)  ) ;
%
alpha_1 = acos( (D^2+L2^2-L3^2)/(2*L2*D) ) ;
alpha_1_deg = rad2deg(alpha_1) ;
%
beta_1 = pi/2-(alpha+alpha_1) ;
beta_2 = beta_0-beta_1 ;
h1 = L2*cos(beta_1)  ;
%
phi_1 = atan2(L2/3,L2)  ;
phi_2 = atan2(L2/3,L2)  ;
%
L2_aux = L2/(2*cos(phi_1)) ;
L3_aux = L3/(2*cos(phi_2)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Robot Description (with floating base)
%
% First Step: a point trough the joint axis in the -zero- configuration
% (usually is the configuration you are going to study, but generally speaking
% could be different)
%
% Virtual Kinematic Chain (parametrizing the floating base)
P1 = [ 0 0 0 ]'  ; % points belonging to the joint axis
P2 = [ 0 0 0 ]'  ;
P3 = [ 0 0 0 ]'  ;
% Real Robot
P4 = [0 -L1 0]'  ;
P5 = [0 -L1 0]'  ;
P6 = P4    +   [-L2_aux*sin(beta_1-phi_1)     -L2_aux*cos(beta_1-phi_1)  0]'  ; % knee
P7 = P5    +   [ L3_aux*sin(beta_2+phi_1)     -L3_aux*cos(beta_2+phi_2)  0]'  ;
% Collecting all the points
P_0 = [P1, P2, P3, P4, P5, P6, P7 ] ;  
%
% Joint Axis 
% floating base
Z1 = [1, 0, 0]'   ;  
Z2 = [0, 1, 0]'   ;
Z3 = [0, 0, 1]'   ;
% Real Robot
Z4 = [0, 0, 1]'   ;
Z5 = [0, 0, 1]'   ;
Z6 = [0, 0, 1]'   ;
Z7 = [0, 0, 1]'   ;
%
Z_0 = [Z1, Z2, Z3, Z4, Z5, Z6, Z7 ]   ; % Collecting all the joint axis versors
%
% End-Effector Positions
%
p_EE_C1 = P4    +   [-L2*sin(beta_1)     -L2*cos(beta_1)  0]'  ; %  First Contact
p_EE_C2 = P5    +   [ L3*sin(beta_2)     -L3*cos(beta_2)  0]'  ; %  Second Contact
%
% Contact Frames w.r.t. {A} in the -zero- configuration
EE_c_0(:,:,1) = homogeneous( eye(3) , p_EE_C1 );
EE_c_0(:,:,2) = homogeneous( eye(3) , p_EE_C2 );
%
% Configuration definition
q_0 = zeros(1,size(P_0,2)) ;
joint_type =  [ 1 1 0 ...  % Virtual Kinematic Chain
                  0 0 0 0 ] % Real Robot  % 1 => prismatic; 0 => revolute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial Forces Computation
d1 = norm(p_EE_C1(1)) ; %norm(p_EE_G1(1)-p_EE_C1(1)) ;
d2 = norm(p_EE_C2(1)) ;
fc2_y_0 = mg_0*d1/(d1+d2); %  contact force from the robot to the environment
fc1_y_0 = - ((-mg_0)+fc2_y_0)
fc2_y_0
% Contact Force Pre-load Definition 
g_0  = [0   mg_0 ]'  ;    %  
fc_0 = [0  fc1_y_0 , 0  fc2_y_0 ]'  ; % 
%
% exerted by the robot on the world
lambda_0 = [fc_0;  -g_0] ; 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Connectivity Matrix Definition
% in this framework the conectivity matrix is 
% Cc \in R^{ c \times #q }
% c = number of contact frames == size(EE_C,3)
% #q = number of joints == size(P,2)
% for each row, the i-th element is 1 (true) 
% if the corresponding i-th joint belongs to the chain,
% zero (false) otherwise
Cc =[ 1 1 1  1 0 1 0 ;...
      1 1 1  0 1 0 1 ] ;
vkc_dim = 3 ;
B_i = [1 0 0 0 0 0; 0 1 0 0 0 0 ]'  ; 
B_c = [B_i, B_i] ;
cf_dim = [2 2] ; 
% Masses
m = zeros(1,size(P_0,2)) ;
m(3) = 50 ;
g_m_0 = eye(4) ;
B_g = B_i(1:3,:) ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Forward Kinematic: Computation Example 
% Example of forward kinematic for serial chains
EE_c1_q = fw_kin_serial( [P1 P2 P3 P4 P6 ], ...
             [Z1 Z2 Z3 Z4 Z6] , EE_c_0(:,:,1)  , [0 0 0  0 0] , [1 1 0  0 0] ) 

EE_c2_q = fw_kin_serial( [P1 P2 P3 P4 P6 ], ...
             [Z1 Z2 Z3 Z4 Z6] ,  EE_c_0(:,:,2)  , [0 0 0  0 0] , [1 1 0  0 0]  ) 
% Forward Kinematic for the tree
EE_tree = fw_kin_tree(P_0, Z_0, EE_c_0, q_0, Cc, joint_type ) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Differential Kinematics: Computation Examples
% Main matrices computation (Jacobian, Stance... etc...)
%
J_c1_spatial_0 = Spatial_Jacobian_serial(  [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], [0 0 0 0 0] , [1 1 0 0 0] ) %
%
J_c1_body_0 = Body_Jacobian_serial(  [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1), [0 0 0 0 0] , [1 1 0 0 0] )

Spa_J_c_tree = Spatial_Jacobian_tree(P_0, Z_0, q_0, Cc, joint_type ) 
%
Body_J_c_tree = Body_Jacobian_tree(P_0, Z_0, EE_c_0, q_0, Cc,joint_type)
%
Hum_Jac  = Humanoid_Jacobian(P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim ,B_c, cf_dim, joint_type ) 
%
Stance_M = Stance_Matrix(P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim , B_c, cf_dim, joint_type ) 
%
%% Joint Stiffness
kq = 1E4 ;           %  [Nm/rad]
% Contact Stiffness
kc = 1E8 ;%   [N/m]
%
Kc = kc*eye( size(Hum_Jac,1) )  ; 
Kq = kq*eye(size(Hum_Jac,2)) ; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Drivatives
Q_c_1 = D_Jt_f_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0], B_i, fc_0(1:2), joint_type)
%       
Q_c_tree = D_Jt_f_tree( P_0, Z_0, EE_c_0, q_0, Cc, B_c, fc_0, cf_dim , joint_type  )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Derivative Blocks
Us_1 = Us_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Qs_1 = Qs_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Uj_1 = Uj_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Qj_1 = Qj_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
%
Us_tot = Us_tree(P_0, Z_0, EE_c_0, q_0, vkc_dim, Cc, B_c, fc_0, cf_dim , joint_type  )
Qs_tot = Qs_tree(P_0, Z_0, EE_c_0, q_0, vkc_dim, Cc, B_c, fc_0, cf_dim , joint_type  )
Uj_tot = Uj_tree(P_0, Z_0, EE_c_0, q_0, vkc_dim, Cc, B_c, fc_0, cf_dim , joint_type  )
Qj_tot = Qj_tree(P_0, Z_0, EE_c_0, q_0, vkc_dim, Cc, B_c, fc_0, cf_dim , joint_type  )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
% Drivative of the Gravitational Term
%
J_com = J_CoM(P_0, Z_0, g_m_0 , q_0, Cc, m, joint_type )
[ Q_g ] = D_Jt_mg(P_0, Z_0, eye(4), q_0, Cc, m, B_g,-g_0, joint_type )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
FLMM_rig  = FLMM_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
FLMM_comp_rig  = FLMM_compact_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
%
FLMM_d  = FLMM_d_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
FLMM_i  = FLMM_i_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
%
cFLMM_rigid      =  cFLMM_rigid( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc )
cFLMM_comp_rigid =  cFLMM_compact_rigid( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc )
%
Rf   = Rf_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc  )  
Rtau = Rtau_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc )  
Ru   = Ru_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) 
%
%
E  = E_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc   ) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Elastic Robot
FLMM_el      =  FLMM_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
FLMM_comp_el =  FLMM_compact_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)

F_d_el = FLMM_d_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
F_i_el = FLMM_i_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)

cFLMM_el      =  cFLMM_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
cFLMM_comp_el =  cFLMM_compact_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
%
R_fc_el  = Rf_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_tau_el = Rtau_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_u_el   = Ru_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_q_el   = Rq_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
%
E_el = E_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq  )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
sigma_leq = sigma_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim  ) ;
%
V_c1 = V_i( fc_0(1:2), ...
                     normals_vect(1:2) ,...
                     mu_vect(1), f_min_vect(1), f_max_vect(1) ) ;

V_0 = V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim )  %, E_el )
% 
grad_V    = D_V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim,  E_el )
Hessian_V = H_V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim , E_el  )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Optimization 
%
% [fc_opt, y,V_opt,V_0, exitflag,output, elapsed_time   ] = ...
%     V_optimal_minunc( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% %  wait = waitforbuttonpress ;
% [fc_opt, y,V_opt,V_0,exitflag,output, elapsed_time  ] = ...
%     V_optimal_minunc_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
%  wait = waitforbuttonpress ;
% 
[fc_opt, y,V_opt_mincon_1,V_0,exitflag,output, elapsed_time, sigma_leq , lambda,grad,hessian   ] = ...
    V_optimal_mincon( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
%  wait = waitforbuttonpress ;
%  
% [fc_opt, y,V_opt_mincon,V_0,exitflag,output, elapsed_time , sigma_leq , lambda,grad,hessian  ] = ...
%     V_optimal_mincon_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% % 
%wait = waitforbuttonpress ;
%%
% percent_decrease  = 1E-30 ;
% max_iter          = 1000  ;
% norm_grad         = 1E-10 ; 
% [fc_opt_newton, y_opt, V_opt, V_0, exit_tests , elapsed_time, sigma_leq, grad_opt, Hessian_opt, V_vect] = ...
%     V_optimal_Newton( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el , max_iter , norm_grad) ;
% %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
% Plots
% Plotting initial Configuration
plot_Humanoid( P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim, joint_type )
plot_Masses(P_0, Z_0, eye(4), q_0, Cc, m, joint_type )
hold on
plot([EE_c_0(1,4,1); EE_c_0(1,4,2)], [EE_c_0(2,4,1); EE_c_0(2,4,2)] ,'k', 'LineWidth',2 )
%
%%
% Computing the Final Configuration
d_qr_opt = pinv(R_fc_el,1E-7)*(fc_opt-fc_0) %fc_opt %fc_opt_newton % Rf
d_u_opt  = R_u_el * d_qr_opt
d_q_opt  = R_q_el * d_qr_opt
d_uq_opt = [ d_u_opt ; d_q_opt ]
% Plotting final Configuration
plot_Humanoid( P_0, Z_0, EE_c_0, d_uq_opt, Cc, vkc_dim, joint_type )
plot_Masses(P_0, Z_0, eye(4), d_uq_opt, Cc, m, joint_type )
hold on
EE_final = fw_kin_tree(P_0, Z_0, EE_c_0, d_uq_opt, Cc, joint_type ) ;
plot([EE_final(1,4,1); EE_final(1,4,2)], [EE_final(2,4,1); EE_final(2,4,2)] ,'k', 'LineWidth',2 )


 
 
 
return
















