% V_grad_tot
%
%   Computes the derivative of the function V (to be minimized) for the current contact force distribution.
%   Friction cone, minimum and maximum normal force are considered
%
%   Syntax: [ V_final ] = V_tot( f_c, normals, mu, f_min, f_max , cf_dim )
%
%   Inputs:
%   f_c     is a vector collecting all the current contact forces
%       Only linear forces are considered (f_ci is \in R^{3}, or \in R^{2} for planar problems). 
%   normals is a vector collecting di directions of all the normals a the contact points
%       (n_i is \in R^{3}, or \in R^{2} for planar problems). 
%   mu    is the vector collecting all the friction coefficient at the contacts
%   f_min is the vector collecting all the minimum normal forces admitted at the contacts
%   f_max is the vector collecting all the maximum normal forces admitted at the contacts
%
% Outputs:
% V_grad_tot is the derivative of the fucntion V relative to the i-th contact
%
function [ V_grad_final ] = V_grad_tot( f_c, normals, mu, f_min, f_max , cf_dim, E, fc_0 )
%
start_i = zeros(size(cf_dim)); % index of start of the fc_i vector
end_i = zeros(size(cf_dim)); % index of end of the fc_i vector
%
start_i(1) = 1 ;
for j = 2:length(cf_dim)
    start_i(j) = start_i(j-1)+cf_dim(j-1); 
end
for j = 1:length(cf_dim)
    end_i(j) = start_i(j) + cf_dim(j)-1 ;
end
V = zeros(2,length(cf_dim)); %[];
for i = 1: length(cf_dim)
   V(:,i) = V_grad_contact( f_c(start_i(i):end_i(i)), ...
                     normals(start_i(i):end_i(i)),...
                     mu(i), f_min(i), f_max(i), ...
                     E(start_i(i):end_i(i),:), fc_0(start_i(i):end_i(i)) ) ;
end
%
V_grad_final = sum(V')' ;
% 
% V_0 = V_tot( f_c, normals, mu, f_min, f_max , cf_dim ) ; 
% y0 = [0 ; 0] ;
% h=  sqrt(eps);
% y_h1 = y0 ;
% y_h1(1) = y_h1(1) +h ;
% V_h1 = V_tot( f_c+E*y_h1, normals, mu, f_min, f_max , cf_dim );
% d_V_1 = (V_h1- V_0)/h ;
% %
% y_h2 = y0 ;
% y_h2(2) = y_h2(2) + h ;
% V_h2 = V_tot( f_c+E*y_h2, normals, mu, f_min, f_max , cf_dim );
% d_V_2 = (V_h2- V_0)/h ;
% %
% V_grad_final = [ d_V_1  d_V_2]' ;
%
end
%
