%
clear all 
clc
format short
%
digits 512
%
alpha_deg_0 = 10
alpha = deg2rad(alpha_deg_0) ;
%
mu_1 =   1 
mu_2 =   .3
%
mg_0 = - 500 ;
L1 = 0.5  ; % Link Lengths 
L2 = 0.5  ;
L3 = 0.5  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parametrized initial robot condiguration
beta_deg_0 = 60 ;
beta_0 = deg2rad(beta_deg_0) ;
D = sqrt( L2^2+ L3^2 -2*L2*L3*cos(beta_0)  ) ;
%
alpha_1 = acos( (D^2+L2^2-L3^2)/(2*L2*D) ) ;
alpha_1_deg = rad2deg(alpha_1) ;
%
beta_1 = pi/2-(alpha+alpha_1) ;
beta_2 = beta_0-beta_1 ;
h1 = L2*cos(beta_1)  ;
%
phi_1 = atan2(L2/3,L2)  ;
phi_2 = atan2(L2/3,L2)  ;
%
L2_aux = L2/(2*cos(phi_1)) ;
L3_aux = L3/(2*cos(phi_2)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%
f_min = 0 ;   % [N] minimum normal contact force allowable
f_max_leg = abs(mg_0) ; % [N] maximum normal contact force allowable 
%
h = sqrt(eps) ; 
%
kq = 1000 ;           % 
kc = 1000000000000  ; % 
% 
P1 = [ 0 0 0 ]'  ; % points belonging to the joint axis
P2 = [ 0 0 0 ]'  ;
P3 = [ 0 0 0 ]'  ;
P4 = [0 -L1 0]'  ;
P5 = [0 -L1 0]'  ;
P6 = P4    +   [-L2_aux*sin(beta_1-phi_1)     -L2_aux*cos(beta_1-phi_1)  0]'  ; % knee
P7 = P5    +   [ L3_aux*sin(beta_2+phi_1)     -L3_aux*cos(beta_2+phi_2)  0]'  ;
%
P = [P1, P2, P3, P4, P5, P6, P7 ] ;  % Collecting all the points
%
Z1 = [1, 0, 0]'   ;  % Joint axis versors
Z2 = [0, 1, 0]'   ;
Z3 = [0, 0, 1]'   ;
Z4 = [0, 0, 1]'   ;
Z5 = [0, 0, 1]'   ;
Z6 = [0, 0, 1]'   ;
Z7 = [0, 0, 1]'   ;
%
Z = [Z1, Z2, Z3, Z4, Z5, Z6, Z7 ]   ; % Collecting all the joint axis versors
%
% End-Effector Positions
EE_B  = [0 0 0]' ;  % Floating base
EE_G1 = [0 0 0]' ;  % Mass 
%
EE_C1 = P4    +   [-L2*sin(beta_1)     -L2*cos(beta_1)  0]'  ; %  First Contact
EE_C2 = P5    +   [ L3*sin(beta_2)     -L3*cos(beta_2)  0]'  ; %  Second Contact
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial Forces Computation
d1 = norm(EE_G1(1)-EE_C1(1)) ;
d2 = norm(EE_G1(1)-EE_C2(1)) ;
fc2_y_0 = mg_0*d1/(d1+d2); %  contact force from the robot to the environment
fc1_y_0 = - ((-mg_0)+fc2_y_0)
fc2_y_0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
joint_type_virtual = [1 1 0] ; % joint types of the Virtual Kinematic Chain % 1 = prismatic joint
joint_type_real =  [0 0] ;     % joint types of the Real Kinematic Chain    % 0 = revolute joint
joint_type = [joint_type_virtual, joint_type_real, joint_type_real] ;
%
% Contact Force Pre-load Definition 
g_0  = [0   mg_0 ]'  ;    %  
fc_0 = [0  fc1_y_0 , 0  fc2_y_0 ]'  ; % 
%
lambda = [fc_0;  -g_0] ; % exerted by the robot
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Spatial Jacobians of relevant frames
%
J_a_b_comp = spatial_WB_Jacobian([P1 P2 P3],[Z1 Z2 Z3], [1 1 0], [1:3] ) ; % spatial_WB_Jacobian( P, Z, joint_type, joint_chain )
J_a_g1_comp = J_a_b_comp   ;
J_a_c1_comp = spatial_WB_Jacobian([P1 P2 P3 P4 P6 ],[Z1 Z2 Z3 Z4 Z6], [1 1 0 0 0], [1:5] )  ;
J_a_c2_comp = spatial_WB_Jacobian([P1 P2 P3 P5 P7 ],[Z1 Z2 Z3 Z5 Z7], [1 1 0 0 0], [1:5] )  ;
%
% According dimensions to the problem
J_a_b_comp  = [J_a_b_comp, zeros(6,4)] ;
J_a_g1_comp = [J_a_g1_comp, zeros(6,4)] ;
J_a_c1_comp = [J_a_c1_comp(:,1:4), zeros(6,1),J_a_c1_comp(:,end),zeros(6,1)] ;
J_a_c2_comp = [J_a_c2_comp(:,1:3), zeros(6,1),J_a_c2_comp(:,4),zeros(6,1), J_a_c2_comp(:,5) ] ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
eye4 = eye(4); 
%
g_ab_0  = eye4 ;  % Initial Configuration of relevant frames
g_ag1_0 = eye4 ;
g_ac1_0 = eye4 ;
g_ac2_0 = eye4 ;
%
g_ab_0(1:3,4)  = EE_B   ;
g_ag1_0(1:3,4) = EE_G1  ;
g_ac1_0(1:3,4) = EE_C1  ;
g_ac2_0(1:3,4) = EE_C2  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Body Jacobians
J_b_b_comp   = ad2(ihomogeneous(g_ab_0))*J_a_b_comp ;
J_g1_g1_comp = ad2(ihomogeneous(g_ag1_0))*J_a_g1_comp  ;
%
J_c1_c1_comp = ad2(ihomogeneous(g_ac1_0))*J_a_c1_comp  ; 
J_c2_c2_comp = ad2(ihomogeneous(g_ac2_0))*J_a_c2_comp  ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
B_b_T  = [1 0 0 0 0 0; 0 1 0 0 0 0 ;  0 0 0 0 0 1 ] ;
B_g1_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ;
B_c1_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ; % hard finger
B_c2_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ;
%
J_b_b   = B_b_T*J_b_b_comp      ;
J_g1_g1 = B_g1_T*J_g1_g1_comp   ;
J_c1_c1 = B_c1_T*J_c1_c1_comp   ;
J_c2_c2 = B_c2_T*J_c2_c2_comp   ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
S_g_T = J_g1_g1(:,1:3) ;
S_g   = S_g_T' ;
J_g   = J_g1_g1(:,4:end) ;
%
J_c_n = [J_c1_c1; J_c2_c2 ]  ;
S_c_T = J_c_n(:,1:3)    ;
S_c = S_c_T' ;
J_c   = J_c_n(:,4:end)  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Derivatives 
%
Q_c1 = D_J_ci(J_a_c1_comp, g_ac1_0 ,  B_c1_T,  fc_0(1:2)) ; 
Q_c2 = D_J_ci(J_a_c2_comp, g_ac2_0 ,  B_c2_T,  fc_0(3:4)) ; 
%
% Derivative of the Gravitational Term
% 
tau_g_0 = (B_g1_T*ad2(ihomogeneous(g_ag1_0))*J_a_g1_comp)'*lambda(5:6) ;
D_g = [] ;
for i= 1:3
%    
q_temp = [ 0 0 0 ];
q_temp(i) =   q_temp(i)+h ;
%
J_s_g1_temp = singleSerialJacobian(P(:,[1:3]),Z(:,[1:3]),q_temp,[1:3], [ 1 1 0]) ;
% 
J_s_g1_temp = [J_s_g1_temp, zeros(6,4)] ;
g_ag1_temp = forward_kinematics_NOSYM(P(:,[1:3]),Z(:,[1:3]),...
        g_ag1_0,q_temp([ 1:3 ]),[1 2])   ; 
g_ag1_temp(1:3,1:3) = eye(3) ;       % the frame does not rotate, can only translate
tau_g_i =(B_g1_T *  ad2( ihomogeneous(g_ag1_temp))*J_s_g1_temp  )'*lambda(5:6)  ; 
%tau_g_i = [ tau_g_i ; 0 ; 0]
    D_g = [D_g, (tau_g_i-tau_g_0)./h ] ;
end
D_g = [D_g, zeros(7,4)] ;
%
Q_tot =  Q_c1 + Q_c2 ;
%
U_s = Q_tot(1:3,1:3) ;
U_j = Q_tot(4:end,1:3) ;
Q_s = Q_tot(1:3,4:end);
Q_j = Q_tot(4:end,4:end) ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sizes
s_fc  = size(J_c,1) ; 
s_tau = size(J_c,2) ; 
s_u  = size(S_g,1) ; 
s_wb = size(S_g,1) ; 
s_q  = size(J_c,2) ; 
%
Kc = kc*eye(s_fc)  ; 
Kq = kq*eye(s_q) ; 
%
%
FLMM_1 = FLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
cFLMM_1 =  cFLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
FLMM_2 = FLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
cFLMM_2 = cFLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
R_f = Rf_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
R_u = Ru_rigid( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc )  ;
R_tau = Rtau_rigid( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
%
R_f_tot = R_f ;
R_f_filtr = round(R_f_tot*10^(-6))*10^6 ;
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Contact Force Optimization
%
% Normal Contact Vector
n_c1 = [0 -1]' ;
n_c2 = [cos( alpha-pi/2) ; sin(alpha-pi/2)  ]  ;
%
check_angle = rad2deg(acos(n_c1'*n_c2 ) ) ;
%
%
E = orth(R_f_filtr) %  
%
cvx_begin               % quiet % non-verbose method
cvx_precision  high     % low % quality solution option
%   cvx_solver sedumi   % alternative solver
variable y_cvx(size(E,2))
%
d_fc = E*y_cvx ;
fc_ott = fc_0+ d_fc ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
sigma_c1_frict = -( cos(atan(mu_1))*norm( fc_ott(1:2)) - fc_ott(1:2)'*n_c1 );
sigma_c2_frict =-( cos(atan(mu_2))*norm( fc_ott(3:4)) - fc_ott(3:4)'*n_c2 );
%
sigma_c1_min = -(f_min - ( fc_ott(1:2))'*n_c1 );
sigma_c2_min = -(f_min - ( fc_ott(3:4))'*n_c2 );
%
sigma_c1_max = -(-f_max_leg + ( fc_ott(1:2))'*n_c1 );
sigma_c2_max = -(-f_max_leg + ( fc_ott(3:4))'*n_c2 );
%
sigma_c1_ott = [pow_p(sigma_c1_frict,-2), pow_p(sigma_c1_min,-2), pow_p(sigma_c1_max,-2)] ;
sigma_c2_ott = [pow_p(sigma_c2_frict,-2), pow_p(sigma_c2_min,-2), pow_p(sigma_c2_max,-2)] ;
% %
sigma_fc_tot = [ sigma_c1_ott, sigma_c2_ott  ]' ;
% 
V_fc_tot = sum((sigma_fc_tot)) ; %norm(sigma_c1_min)%%sum(pow_p( sigma_fc_tot(1), -2 )) ;%
%
%
minimize( V_fc_tot )   
%        subject to
%         sigma_fc_tot >= 0 ;
cvx_end
%
% Recomputation of the last step results
%
d_fc = E*y_cvx ;
fc_ott = fc_0+ d_fc ;
%
sigma_c1_frict = cos(atan(mu_1))*norm( fc_ott(1:2)) - fc_ott(1:2)'*n_c1 ;
sigma_c2_frict = cos(atan(mu_2))*norm( fc_ott(3:4)) - fc_ott(3:4)'*n_c2 ;
%
sigma_c1_min = f_min - ( fc_ott(1:2))'*n_c1 ;
sigma_c2_min = f_min - ( fc_ott(3:4))'*n_c2 ;
%
sigma_c1_max = -f_max_leg + ( fc_ott(1:2))'*n_c1 ;
snapnow;
sigma_c2_max = -f_max_leg + ( fc_ott(3:4))'*n_c2 ;
%
sigma_c1_ott = [sigma_c1_frict, sigma_c1_min, sigma_c1_max] ;
sigma_c2_ott = [sigma_c2_frict, sigma_c2_min, sigma_c2_max] ;
%
sigma_fc_tot = [ sigma_c1_ott, sigma_c2_ott  ]' ;
% 
V_fc_tot = sum(exp(sigma_fc_tot)) ;
%
d_fc_tot_wb = E*y_cvx ;
fc_tot_wb = fc_0 + d_fc_tot_wb  % 
%
d_q = pinv(-R_f_tot)*d_fc_tot_wb
d_u = -R_u*d_q
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Section
P_Plot = [EE_G1, P4 , P5, P6, P7]  ;
%
Z_plot = [ Z3, Z4 Z5 Z6 Z7]    ;
EE_plot = [ EE_C1 , EE_C2  ]  ;
%
close all
figure
grid on
axis equal
axis( [ -.35 .55  -1 .1 ] ) % 
%  
plot_Robot(P_Plot, Z_plot, EE_plot, [0 0 0 0 0], [0 0 0 0 0], [1 2 4], [1 3 5]);
plot_Object(homogeneous( eye(3) , EE_G1 ) ,.05,0,'Sphere');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Plot of the final configuration
%
EE_G1_final =  forward_kinematics_NOSYM( [P1 P2 P3] , [Z1 Z2 Z3],  homogeneous( eye(3) , EE_G1 )  , d_u , [1 2] ) ;
EE_G1_final = EE_G1_final(1:3,4)  ;
% 
P3_final =  forward_kinematics_NOSYM( [P1 P2 ] , [Z1 Z2 ],  homogeneous( eye(3) , P3 )  , d_u(1:2) , [1 2] ) ;
%
P4_final = forward_kinematics_NOSYM( [P1 P2 P3] ,...
    [Z1 Z2 Z3 ],  homogeneous( eye(3) , P4 )  , [ d_u; ] , [ 1 2 ] ) ;
P4_final = P4_final(1:3,4)  ;
%
P5_final  = forward_kinematics_NOSYM( [P1 P2 P3 ] ,...
    [Z1 Z2 Z3 ],  homogeneous( eye(3) , P5 )  , [ d_u; ] , [ 1 2 ] );
P5_final = P5_final(1:3,4)  ;
%
P6_final  = forward_kinematics_NOSYM( [P1 P2 P3 P4 ] ,...
    [Z1 Z2 Z3 Z4 ],  homogeneous( eye(3) , P6 )  , [ d_u; d_q(1) ] , [ 1 2 ] );
P6_final = P6_final(1:3,4)  ;
%
P7_final  = forward_kinematics_NOSYM( [P1 P2 P3 P5 ] ,...
    [Z1 Z2 Z3 Z5 ],  homogeneous( eye(3) , P7 )  , [ d_u; d_q(2) ] , [ 1 2 ] );
P7_final = P7_final(1:3,4)  ;
%  
% %
d_C = S_c_T*d_u+J_c*d_q ;
EE_C1_final = [ EE_C1(1:2) + d_C(1:2);0 ] ;
EE_C2_final = [ EE_C2(1:2) + d_C(3:4);0 ] ;
EE_plot_final = [ EE_C1_final , EE_C2_final  ]  ;
%
P_Plot_final = [EE_G1_final,  P4_final , P5_final, P6_final, P7_final]   ;
%
figure
grid on
axis equal
axis( [ -.35 .55  -1 .1 ] )
plot_Robot(P_Plot_final ,Z_plot, EE_plot_final,[0 0 0 0 0], [0 0 0 0 0], [1 2 4], [1 3 5]);
plot_Object( homogeneous( eye(3) , EE_G1_final ) ,.05,0,'Sphere');
%




























