%
% This is an example explointing the funtionalities of the toolbox
%
clear all 
close all
clc
format short
%
% TODO
% 1) figura pdf di riferimento con riportati sopra i parametri utilizzati
% 2) readme file che spiega come dalla figura di riferimento si passa alla
%    scrittura del codice riportato
% 3) file uguale ma senza parametrizzazione => compact version
% 4) Impostazione automatica della floating base
% 5) Riduzione del problema a planare (vedi parte di Nicola)
% 6) introdurre riferimenti a copyright all'inizio di ogni file
% 7) introduzione della dinamica come wrench esterno

% 8) Altro esempio con piede parametrizzato
% 9) Altro esempio parametrizzato con mano su muro
%%%%%%%%%%%%%%%
% Some useful parameters of the problem
% Weight of the Robot
mg_0 = - 500 ;  % [kg]
% Inclination of the surface
alpha_deg_0 = 10 % (deg)
alpha = deg2rad(alpha_deg_0) ; % (rad)
%
n_c1 = [0 -1]' ;
n_c2 = [cos( alpha-pi/2) ; sin(alpha-pi/2)  ]  ;
normals_vect = [n_c1; n_c2] ;
% Fricition Coefficients on the contacts
mu_1 =  1 
mu_2 =  1
mu_vect = [mu_1, mu_2] ;
%
f_min = 0 ;           % [N] minimum normal contact force allowable
f_max_leg = abs(mg_0) ; % [N] maximum normal contact force allowable 
%
f_min_vect = [f_min, f_min];
f_max_vect = [f_max_leg, f_max_leg];
%
% Link Lengths 
L1 = 0.5  ; % [m]
L2 = 0.5  ; % [m]
L3 = 0.5  ; % [m]
%
% Joint Stiffness
kq = 1E4 ;           %  [Nm/rad]
% Contact Stiffness
kc = 1E12 ;% !E12  1000000000000  ; %  [N/m]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parametrized initial robot condiguration
beta_deg_0 = 60 ;
beta_0 = deg2rad(beta_deg_0) ;
D = sqrt( L2^2+ L3^2 -2*L2*L3*cos(beta_0)  ) ;
%
alpha_1 = acos( (D^2+L2^2-L3^2)/(2*L2*D) ) ;
alpha_1_deg = rad2deg(alpha_1) ;
%
beta_1 = pi/2-(alpha+alpha_1) ;
beta_2 = beta_0-beta_1 ;
h1 = L2*cos(beta_1)  ;
%
phi_1 = atan2(L2/3,L2)  ;
phi_2 = atan2(L2/3,L2)  ;
%
L2_aux = L2/(2*cos(phi_1)) ;
L3_aux = L3/(2*cos(phi_2)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Robot Description (with floating base)
%
% First Step: a point trough the joint axis in the -zero- configuration
% (usually is the configuration you are going to study, but generally speaking
% could be different)
%
% Virtual Kinematic Chain (parametrizing the floating base)
P1 = [ 0 0 0 ]'  ; % points belonging to the joint axis
P2 = [ 0 0 0 ]'  ;
P3 = [ 0 0 0 ]'  ;
% Real Robot
P4 = [0 -L1 0]'  ;
P5 = [0 -L1 0]'  ;
P6 = P4    +   [-L2_aux*sin(beta_1-phi_1)     -L2_aux*cos(beta_1-phi_1)  0]'  ; % knee
P7 = P5    +   [ L3_aux*sin(beta_2+phi_1)     -L3_aux*cos(beta_2+phi_2)  0]'  ;
% Collecting all the points
P_0 = [P1, P2, P3, P4, P5, P6, P7 ] ;  
%
% Joint Axis 
% floating base
Z1 = [1, 0, 0]'   ;  
Z2 = [0, 1, 0]'   ;
Z3 = [0, 0, 1]'   ;
% Real Robot
Z4 = [0, 0, 1]'   ;
Z5 = [0, 0, 1]'   ;
Z6 = [0, 0, 1]'   ;
Z7 = [0, 0, 1]'   ;
%
Z_0 = [Z1, Z2, Z3, Z4, Z5, Z6, Z7 ]   ; % Collecting all the joint axis versors
%
% End-Effector Positions
p_EE_B  = [0 0 0]' ;  % Floating base
p_EE_G1 = [0 0 0]' ;  % Mass 
%
p_EE_C1 = P4    +   [-L2*sin(beta_1)     -L2*cos(beta_1)  0]'  ; %  First Contact
p_EE_C2 = P5    +   [ L3*sin(beta_2)     -L3*cos(beta_2)  0]'  ; %  Second Contact
%
% Contact Frames w.r.t. {A} in the -zero- configuration
EE_c_0(:,:,1) = homogeneous( eye(3) , p_EE_C1 );
EE_c_0(:,:,2) = homogeneous( eye(3) , p_EE_C2 );
% Floating Base frame {B} w.r.t. {A} in the -zero- configuration
EE_B = eye(4) ;
%
% Configuration definition
q_0 = zeros(1,size(P_0,2)) ;
joint_type =  [ 1 1 0 ...  % Virtual Kinematic Chain
                  0 0 0 0 ] % Real Robot  % 1 => prismatic; 0 => revolute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial Forces Computation
d1 = norm(p_EE_G1(1)-p_EE_C1(1)) ;
d2 = norm(p_EE_G1(1)-p_EE_C2(1)) ;
fc2_y_0 = mg_0*d1/(d1+d2); %  contact force from the robot to the environment
fc1_y_0 = - ((-mg_0)+fc2_y_0)
fc2_y_0
% Contact Force Pre-load Definition 
g_0  = [0   mg_0 ]'  ;    %  
fc_0 = [0  fc1_y_0 , 0  fc2_y_0 ]'  ; % 
%
% exerted by the robot on the world
lambda_0 = [fc_0;  -g_0] ; 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Connectivity Matrix Definition
% in this framework the conectivity matrix is 
% Cc \in R^{ c \times #q }
% c = number of contact frames == size(EE_C,3)
% #q = number of joints == size(P,2)
% for each row, the i-th element is 1 (true) 
% if the corresponding i-th joint belongs to the chain,
% zero (false) otherwise
Cc =[ 1 1 1  1 0 1 0 ;...
      1 1 1  0 1 0 1 ] ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Forward Kinematic: Computation Example 
% Example of forward kinematic for serial chains
EE_c1_q = fw_kin_serial( [P1 P2 P3 P4 P6 ], ...
             [Z1 Z2 Z3 Z4 Z6] , EE_c_0(:,:,1)  , [0 0 0  0 0] , [1 1 0  0 0] ) 

EE_c2_q = fw_kin_serial( [P1 P2 P3 P4 P6 ], ...
             [Z1 Z2 Z3 Z4 Z6] ,  EE_c_0(:,:,2)  , [0 0 0  0 0] , [1 1 0  0 0]  ) 
% Forward Kinematic for the tree
EE_tree = fw_kin_tree(P_0, Z_0, EE_c_0, q_0, Cc, joint_type ) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Differential Kinematics: Computation Examples
% Main matrices computation (Jacobian, Stance... etc...)
Cc =[ 1 1 1  1 0 1 0 ;...
      1 1 1  0 1 0 1 ] ;
vkc_dim = 3 ;
B_i = [1 0 0 0 0 0; 0 1 0 0 0 0 ]'  ; 
B_c = [B_i, B_i] ;
cf_dim = [2 2] ; 
m = zeros(1,size(P_0,2)) ;
m(3) = 50 ;
g_m_0 = eye(4) ;
B_g = B_i(1:3,:) ;
%
J_c1_spatial_0 = Spatial_Jacobian_serial(  [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], [0 0 0 0 0] , [1 1 0 0 0] ) %
J_c2_spatial_0 = Spatial_Jacobian_serial(  [P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], [0 0 0 0 0] , [1 1 0 0 0] ) %
%
J_c1_body = ad2(ihomogeneous(EE_c_0(:,:,1)))*J_c1_spatial_0          
J_c2_body = ad2(ihomogeneous(EE_c_0(:,:,2)))*J_c2_spatial_0     
%
J_c1_body_0 = Body_Jacobian_serial(  [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1), [0 0 0 0 0] , [1 1 0 0 0] )
       %
J_c2_body_0 = Body_Jacobian_serial(  [P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0] , [1 1 0 0 0] ) 
%
Spa_J_c_tree = Spatial_Jacobian_tree(P_0, Z_0, q_0, Cc, joint_type ) 
%
Body_J_c_tree = Body_Jacobian_tree(P_0, Z_0, EE_c_0, q_0, Cc,joint_type)
%
vkc_dim = 3 ;
Hum_Jac  = Humanoid_Jacobian(P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim ,B_c, cf_dim, joint_type ) 
Stance_M = Stance_Matrix(P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim , B_c, cf_dim, joint_type ) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Drivatives
B_i = [1 0 0 0 0 0; 0 1 0 0 0 0 ]'  ; 
B_c = [B_i, B_i] ;

cf_dim = [2 2] ; 

Q_1 = D_Jt_f_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0], B_i, fc_0(1:2), joint_type)
       
Q_2 = D_Jt_f_serial( [P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0], B_i, fc_0(3:4), joint_type)
%       
Q_c_tree = D_Jt_f_tree( P_0, Z_0, EE_c_0, q_0, Cc, B_c, fc_0, cf_dim , joint_type  )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Derivative Blocks
Us_1 = Us_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Qs_1 = Qs_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Uj_1 = Uj_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
Qj_1 = Qj_serial( [P1 P2 P3 P4 P6 ],...
           [Z1 Z2 Z3 Z4 Z6], EE_c_0(:,:,1),  [0 0 0 0 0],...
           3 ,B_i, fc_0(1:2), joint_type )
%
%
Us_2 = Us_serial([P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0], ...
           3, B_i, fc_0(3:4), joint_type)
Qs_2 = Qs_serial( [P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0], ...
           3, B_i, fc_0(3:4), joint_type)
Uj_2 = Uj_serial( [P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0], ...
           3, B_i, fc_0(3:4), joint_type)
Qj_2 = Qj_serial([P1 P2 P3 P5 P7 ],...
           [Z1 Z2 Z3 Z5 Z7], EE_c_0(:,:,2), [0 0 0 0 0], ...
           3, B_i, fc_0(3:4), joint_type)
%
%
Us_tot = Us_tree(P_0, Z_0, EE_c_0, q_0, 3, Cc, B_c, fc_0, cf_dim , joint_type  )
Qs_tot = Qs_tree(P_0, Z_0, EE_c_0, q_0, 3, Cc, B_c, fc_0, cf_dim , joint_type  )
Uj_tot = Uj_tree(P_0, Z_0, EE_c_0, q_0, 3, Cc, B_c, fc_0, cf_dim , joint_type  )
Qj_tot = Qj_tree(P_0, Z_0, EE_c_0, q_0, 3, Cc, B_c, fc_0, cf_dim , joint_type  )
%
Q_c_tree
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
% Drivative of the Gravitational Term
%
q_g =  [ 0 0 0 ] ;
joint_gravity = [1 1 0] ;
%
%%%%%%%%%%
J_g_body_0 = Body_Jacobian_serial(  [P1 P2 P3],...
           [Z1 Z2 Z3], eye(4), q_g , joint_gravity )        
tau_g_0 = J_g_body_0'*B_i*(-g_0)
%

%%%%%%%%
%
q_g_2 =  [ 1 1 1 ] ;
%
J_g_spa_0 = Spatial_Jacobian_serial(  [P1 P2 P3 ],...
           [Z1 Z2 Z3 ] , q_g_2 , joint_gravity )

EE_g = fw_kin_serial(  [P1 P2 P3 ],...
           [Z1 Z2 Z3 ] , eye(4) ,q_g_2, joint_gravity ) ;
d_EE_g = EE_g(1:3,4);
EE_g2 = homogeneous(eye(3), d_EE_g) ;
J_g_no_rot_0 = ad2( ihomogeneous(EE_g2) )*J_g_spa_0 ;
tau_g_no_rot_0 = J_g_no_rot_0'*B_i*(-g_0)
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEMP = testing D_g
%
m = zeros(1,size(P_0,2)) ;
m(3) = 50 ;
g_m_0 = eye(4) ;
J_com = J_CoM(P_0, Z_0, g_m_0 , q_0, Cc, m, joint_type )
%
% rivedere
%Q_g = D_Jt_mg( [P1 P2 P3], [Z1 Z2 Z3], eye(4), [0 0 0], B_i, -g_0 , joint_type  )
B_g = B_i(1:3,:) ;
%  
[ Q_g ] = D_Jt_mg(P_0, Z_0, eye(4), q_0, Cc, m, B_g,-g_0, joint_type )
% TODO : CoM location computation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s_fc  = size(Hum_Jac,1) ; 
s_tau = size(Hum_Jac,2) ; 
s_q  = s_tau ; 
%
Kc = kc*eye(s_fc)  ; 
Kq = kq*eye(s_q) ; 
%
% Testing controllable contact forces with LU factorization
FLMM  = FLMM_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
%
% TODO: Check FLMM_compact_rigid
% % TODO: 
% FLMM_d
% FLMM_i
% cFLMM
% R_f % e le altre R
% E

% sigma_ij; V; ottimizzazione con matlab; plots
FLMM_d = FLMM(:, 1: size(FLMM,1)) ;
FLMM_i = FLMM(:, size(FLMM,1)+1:end) 
%
Fd  = FLMM_d_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
Fi  = FLMM_i_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
%
check_1 = max(max(Fd-FLMM_d))
check_2 = max(max(Fi-FLMM_i))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some tests
setup.type = 'crout';
setup.milu = 'col'; %'row';
setup.droptol = 0.1;
[L,U]= ilu(sparse(FLMM_d),setup) ;
FLMM_d_inv = full(inv(U)*inv(L)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
cFLMM  = FLMM_d\FLMM ;
% 
R_f = cFLMM(1:s_fc, end-s_q+1:end) ;
Rf  = Rf_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 
R_f./max(max(R_f))-Rf./max(max(R_f)) ;
%
R_f_temp = R_f;
R_f_temp(~R_f_temp)=inf ;
minimum = min(min(abs(R_f_temp))) ; % minimum non-zero value of the matrix
%
R_f_2 = round(R_f./(minimum))*minimum ;
%
Rf  = Rf_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc  )  
%
check_3 = R_f_2./max(max(R_f_2))-Rf./max(max(R_f_2)) ;
%
Rtau = Rtau_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc )  
%
Ru  = Ru_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Rf
%
E  = E_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc  ) 
FLMM  = FLMM_rigid(  Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc ) ; 

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Elastic Robot
F_d_el = FLMM_d_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
F_i_el = FLMM_i_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
FLMM_el      =  FLMM_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
FLMM_comp_el =  FLMM_compact_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
cFLMM_el      =  cFLMM_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
cFLMM_comp_el =  cFLMM_compact_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
%
R_fc_el = Rf_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_tau_el = Rtau_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_u_el = Ru_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
R_q_el = Rq_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq)
%
E_el = E_elastic( Hum_Jac,  Stance_M , Qj_tot,  Qs_tot, Uj_tot , Us_tot,  Kc , Kq )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
V_c1 = V_i( fc_0(1:2), ...
                     normals_vect(1:2) ,...
                     mu_vect(1), f_min_vect(1), f_max_vect(1) ) ;
V_c2 = V_i( fc_0(3:4), ...
                     normals_vect(3:4) ,...
                      mu_vect(2), f_min_vect(2), f_max_vect(2) ) ;
V_0 = V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim )  %, E_el )
% 
% Optimization
mu_vect(1) = .1 % .01 %.3


% [fc_opt, y,V_opt,V_0, exitflag,output, elapsed_time   ] = ...
%     V_optimal_minunc( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% [fc_opt, y,V_opt,V_0,exitflag,output, elapsed_time  ] = ...
%     V_optimal_minunc_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% % wait = waitforbuttonpress ;
% 
return
[fc_opt, y,V_opt_mincon_1,V_0,exitflag,output, elapsed_time, sigma_leq , lambda,grad,hessian   ] = ...
    V_optimal_mincon( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )

% [fc_opt, y,V_opt_mincon,V_0,exitflag,output, elapsed_time ,lambda,grad,hessian  ] = ...
%     V_optimal_mincon_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% 
%
percent_decrease  = 1E-30 ;
max_iter          = 1000 ;
norm_grad         = 1E-10;
%
 % fc_opt_mincon = fc_opt_mincon_grad
[fc_opt, y_opt, V_opt, V_0, exit_tests , elapsed_time, sigma_leq, grad_opt, Hessian_opt] = ...
    V_optimal_Newton( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el , max_iter , norm_grad)


return
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%














return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
% Plots
% Plotting initial Configuration
plot_Humanoid( P_0, Z_0, EE_c_0, q_0, Cc, vkc_dim, joint_type )
plot_Masses(P_0, Z_0, eye(4), q_0, Cc, m, joint_type )
%
% Computing the Final Configuration
d_qr_opt = pinv(R_fc_el)*fc_opt
d_u_opt = R_u_el * d_qr_opt
d_q_opt = R_q_el * d_qr_opt
d_uq_opt = [ d_u_opt ; d_q_opt ]
% Plotting final Configuration
plot_Humanoid( P_0, Z_0, EE_c_0, d_uq_opt, Cc, vkc_dim, joint_type )
plot_Masses(P_0, Z_0, eye(4), d_uq_opt, Cc, m, joint_type )




return
plot_Humanoid( P_Plot, Z_plot, EE_plot, [0 0 0 0 0], [0 0 0 0 0], [1 2 4], [1 3 5]) ;
 % plot_Object(homogeneous( eye(3) , EE_G1 ) ,.05,0,'Sphere');
plot_Object( eye(4)  ,.05,0,'Sphere');








return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gradient of V
D_V_c_1_form =  D_V_i( fc_0(1:2), normals_vect(1:2),...
    mu_vect(1), f_min_vect(1), f_max_vect(1) , E_el(1:2,:) ) 
D_V_c_2_form =  D_V_i( fc_0(3:4), normals_vect(3:4),...
    mu_vect(2), f_min_vect(2), f_max_vect(2) , E_el(3:4,:) ) 

D_V_c1 = D_V_tot( fc_0(1:2), normals_vect(1:2),...
    mu_vect(1), f_min_vect(1), f_max_vect(1) , 2,  E_el(1:2,:) ) 
D_V_c2 = D_V_tot( fc_0(3:4), normals_vect(3:4),...
    mu_vect(2), f_min_vect(2), f_max_vect(2) , 2, E_el(3:4,:) ) 
%%
fc_0 =  1.3*fc_opt   % 1.02*fc_opt ;
sigma_now =  sigma_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) 


D_V_tot_new = D_V_tot(  fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim,  E_el )
D_V_tot_paper = D_V_y_AB(  fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim,  E_el )
%
% Finite Differences
V_c = V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % V_contact( fc_opt    , n_0 , 1, 0, 500 )
h =  eps^(1/4); % sqrt(eps) ; % 
y_h_1  = [h ; 0] ;
y_h_2  = [0 ; h] ;
f_c_h1 = (fc_0 + E_el*y_h_1) ;
f_c_h2 = (fc_0 + E_el*y_h_2) ;
f_c_mh1 = (fc_0 - E_el*y_h_1) ;
f_c_mh2 = (fc_0 - E_el*y_h_2) ;
f_c_2h1 = (fc_0 + 2*E_el*y_h_1) ;
f_c_2h2 = (fc_0 + 2*E_el*y_h_2) ;
f_c_2mh1 = (fc_0 - 2*E_el*y_h_1) ;
f_c_2mh2 = (fc_0 - 2*E_el*y_h_2) ;
%
V_c_h1 = V_tot( f_c_h1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_h2 = V_tot( f_c_h2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % (f_0 + E_0*y_h_1)
V_c_mh1 = V_tot( f_c_mh1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_mh2 = V_tot( f_c_mh2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; 
%
V_c_2h1 = V_tot( f_c_2h1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_2h2 = V_tot( f_c_2h2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % (f_0 + E_0*y_h_1)
V_c_2mh1 = V_tot( f_c_2mh1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_2mh2 = V_tot( f_c_2mh2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; 
%
D_V_c_1 = (V_c_h1 - V_c)/h ;
D_V_c_2 = (V_c_h2 - V_c)/h ;
D_V_c_forw_eul = [D_V_c_1 ; D_V_c_2]   ; 
%
D_V_c_1_mixed = (V_c_h1 - V_c_mh1)/(2*h) ;
D_V_c_2_mixed = (V_c_h2 - V_c_mh2)/(2*h) ;
D_V_c_mixed_eul = [D_V_c_1_mixed ; D_V_c_2_mixed]   ;
%
D_V_c_5_points_1 =  (-V_c_2h1+8*V_c_h1-8*V_c_mh1+V_c_2mh1)/(12*h) ;
D_V_c_5_points_2 =  (-V_c_2h2+8*V_c_h2-8*V_c_mh2+V_c_2mh2)/(12*h) ;
D_V_c_5_points = [D_V_c_5_points_1 ; D_V_c_5_points_2 ] 
% D_V_tot_new = D_V_tot(  fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim,  E_el )
% D_V_tot_paper = D_V_y_AB(  fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim,  E_el )
% %
return
%%
% Numerical Hessian
fc_0 =  1.36*fc_opt ;  % 1.36514*fc_opt ;  % 1.6*fc_opt ;  % 
V_c = V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % V_contact( fc_opt    , n_0 , 1, 0, 500 )
%
h =  eps^(1/8); % sqrt(eps) ; % 
y_h_1  = [h ; 0] ;
y_h_2  = [0 ; h] ;
y_h_12  = [h ; h] ;
f_c_h1 = (fc_0 + E_el*y_h_1) ;
f_c_h2 = (fc_0 + E_el*y_h_2) ;
f_c_h12 = (fc_0 + E_el*y_h_12) ;
f_c_mh1 = (fc_0 - E_el*y_h_1) ;
f_c_mh2 = (fc_0 - E_el*y_h_2) ;
%
V_c_h1 = V_tot( f_c_h1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_h2 = V_tot( f_c_h2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % (f_0 + E_0*y_h_1)
V_c_h12 = V_tot( f_c_h12, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; % (f_0 + E_0*y_h_1)
%
V_c_mh1 = V_tot( f_c_mh1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );  
V_c_mh2 = V_tot( f_c_mh2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ; 

H_11 = (V_c_h1 -2*V_c +V_c_mh1)/(h^2)
H_22 = (V_c_h2 -2*V_c +V_c_mh2)/(h^2)
H_12 = (V_c_h12 -V_c_h1 -V_c_h2 +V_c)/(h^2) % forward method, not the best, but should work!
H_finite_diff = [H_11, H_12; H_12, H_22]
%  det_H = det(H)
H_new_form = H_V_tot(fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim , E_el  )
H_AB = H_V_tot_AB(fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim , E_el  )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Optimizing using the gradient
% [fc_opt, y,V_opt,V_0, exitflag,output, elapsed_time   ] = ...
% V_optimal_minunc_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
%  
% [fc_opt, y,V_opt,V_0, exitflag,output, elapsed_time   ] = ...
% V_optimal_mincon_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el )
% %
return

%% 
% f_ci = fc_0(3:4)
f_0 =  [2; 1] ; % [123; 675] %
% n_0 = normals_vect(3:4)
n_0 = [1;1]
n_0 = unit(n_0)
% E_0 = E_el(3:4,:)
E_0 =  [1 1; 1 1] ; % [1 5; 34 4]
y_0 =  [ 1 ; 1 ] ; %[0 ; 0 ] %
f_ci = f_0 + E_0*y_0
% f_ci = f_tot_0 ;
h = sqrt(eps) ;
y_h_1 = y_0 + [h ; 0]
y_h_2 = y_0 + [0 ; h]
f_c_h1 = (f_0 + E_0*y_h_1) ;
f_c_h2 = (f_0 + E_0*y_h_2) ;
% 
% Finite Differences
V_ci    =  V_contact( f_ci    , n_0 , 1, 0, 500 )
V_ci_h1 =  V_contact( f_c_h1  , n_0 , 1, 0, 500 )  
V_ci_h2 =  V_contact( f_c_h2  , n_0 , 1, 0, 500 )  % (f_0 + E_0*y_h_1)
%
D_V_c_1 =  (V_ci_h1 - V_ci)/h ;
D_V_c_2 =  (V_ci_h2 - V_ci)/h ;
D_V_c_diff = [D_V_c_1 ; D_V_c_2]
% 
% Formula 1
D_V_c_1_form = D_V_i(    f_ci, n_0,  1, 0, 500 , E_0  ) 
% Formula 1.1
D_V_tot      = D_V_tot(    f_ci, n_0,  1, 0, 500 , 2,  E_0 )
% Formula AB
D_V_c_1_form = D_V_i_AB(    f_ci, n_0,  1, 0, 500 , E_0  ) 
% Formula AB.1
D_V_tot_AB   = D_V_y_AB( f_ci, n_0,  1, 0, 500 , 2,  E_0 )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
f_0 =  [0; 0] ; % [123; 675] %
n_0 = [1;1]
n_0 = unit(n_0)
E_0 =  [1 0; 0 1] ; % [1 5; 34 4]
y_0 =  [ 1 ; 1 ] ; %[0 ; 0 ] %
f_ci = f_0 + E_0*y_0 ;
h = sqrt(eps) ;
y_h_1 = y_0 + [h ; 0] ;
y_h_2 = y_0 + [0 ; h] ;
f_c_h1 = (f_0 + E_0*y_h_1) ;
f_c_h2 = (f_0 + E_0*y_h_2) ;
% 
% Finite Differences
V_ci    =  V_contact( f_ci    , n_0 , 1, 0, 500 ) ;
V_ci_h1 =  V_contact( f_c_h1  , n_0 , 1, 0, 500 ) ; 
V_ci_h2 =  V_contact( f_c_h2  , n_0 , 1, 0, 500 )  ;% (f_0 + E_0*y_h_1)
%
D_V_c_1 =  (V_ci_h1 - V_ci)/h ;
D_V_c_2 =  (V_ci_h2 - V_ci)/h ;
D_V_c_diff = [D_V_c_1 ; D_V_c_2]
% 
% Formula 1
% D_V_c_1_form = D_V_i(    f_ci, n_0,  1, 0, 500 , E_0  ) 
% Formula 1.1
D_V_tot      = D_V_tot(    f_ci, n_0,  1, 0, 500 , 2,  E_0 )
% Formula AB
% D_V_c_1_form = D_V_i_AB(    f_ci, n_0,  1, 0, 500 , E_0  ) 
% Formula AB.1
D_V_tot_AB   = D_V_y_AB( f_ci, n_0,  1, 0, 500 , 2,  E_0 )
%
return














%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coherency Tests 
V_c2 = V_contact( fc_0(3:4), ...
    normals_vect(3:4) ,...
    mu_vect(2), f_min_vect(2), f_max_vect(2) )

V_c2 = V_contact( [0 ; -500], ...
    normals_vect(3:4) ,...
    mu_vect(2), f_min_vect(2), f_max_vect(2) )
%
epsilon = eps^(1/8) ; % 
a = (3/2)*(1/epsilon^4) ;
b = 4*(1/epsilon^3) ;
c = 3/(epsilon^2) ;  %1/(15*epsilon^2); % -5/epsilon^2; % %
%
h = eps^(1/2) ;
sig_temp = -epsilon ;
V_temp_1 =  ( 2*sig_temp^2 )^(-1) 
V_temp_2 =  a*sig_temp^(2) + b*sig_temp + c 
t_1 = (V_temp_1-V_temp_2)/V_temp_1   % relative error
%
sig_temp_ph = sig_temp +h 
V_temp_ph_1 =  ( 2*sig_temp_ph^2 )^(-1) 
V_temp_ph_2 =  a*sig_temp_ph^(2) + b*sig_temp_ph + c
%
D_V_1 = (V_temp_ph_1-V_temp_1)/h
D_V_2 = (V_temp_ph_2-V_temp_2)/h
t_2 = (D_V_1 - D_V_2)/D_V_1   % relative error
%
D_V_form_1 = -(sig_temp)^(-3)
D_V_form_2 = 2*a*(sig_temp)+b 
t_3 = (D_V_form_1 - D_V_form_2)/D_V_form_1
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Verifying the left part sig = -1
sig_temp = -10 ;
V_temp_1 =  ( 2*sig_temp^2 )^(-1) 
%
sig_temp_ph = sig_temp +h 
V_temp_ph_1 =  ( 2*sig_temp_ph^2 )^(-1) 
%
D_V_1 = (V_temp_ph_1-V_temp_1)/h
%
D_V_form_1 = -(sig_temp)^(-3)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Verifying the left part sig = -1
sig_temp = 10 ;
V_temp_2 =  a*sig_temp^(2) + b*sig_temp + c 
%
sig_temp_ph = sig_temp +h 
 V_temp_ph_2 =  a*sig_temp_ph^(2) + b*sig_temp_ph + c
%
D_V_2 = (V_temp_ph_2-V_temp_2)/h
%
D_V_form_2 = 2*a*(sig_temp)+b 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
%%
% Verifying  (d sigma)/(d y)
alpha = 1/sqrt(1+ mu_vect(2)  ^2 )
f_min_0 = f_min_vect(2)
f_max_0 = f_max_vect(2)
%
f_0 = fc_0(3:4)
f_0 = [123; 675]
n_0 = normals_vect(3:4)
n_0 = [1;1]
n_0 = unit(n_0)
E_0 = E_el(3:4,:)
E_0 = [1 5; 34 4]
y_0 =   [23 ; 81] % [0 ; 0 ] %
f_tot_0 = f_0 + E_0*y_0
f_ci = f_tot_0 ;
h = sqrt(eps) ;
y_h_1 = y_0 + [h ; 0]
y_h_2 = y_0 + [0 ; h]
%
%
sigma_1 = alpha * ((f_0 + E_0*y_0)'*(f_0 + E_0*y_0))^(1/2) - (f_0 + E_0*y_0)'*n_0 ;
%
sigma_1_h1 = alpha * ((f_0 + E_0*y_h_1)'*(f_0 + E_0*y_h_1))^(1/2) - (f_0 + E_0*y_h_1)'*n_0 ;
%
sigma_1_h2 = alpha * ((f_0 + E_0*y_h_2)'*(f_0 + E_0*y_h_2))^(1/2) - (f_0 + E_0*y_h_2)'*n_0 ;
%
D_sig_1_1 = (sigma_1_h1-sigma_1)/h ; 
D_sig_1_2 = (sigma_1_h2-sigma_1)/h ;
D_sig_1 = [D_sig_1_1 ; D_sig_1_2]
%
D_sigma_1_form =  alpha*((f_ci)'*(f_ci))^(-1/2)   ...
            * ((E_0')*f_ci ) - ( E_0'*n_0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma_2 =  - (f_0 + E_0*y_0)'*n_0 + f_min_0 ;
sigma_2_h1 =  - (f_0 + E_0*y_h_1)'*n_0 + f_min_0 ;
sigma_2_h2 =  - (f_0 + E_0*y_h_2)'*n_0 + f_min_0 ;
D_sig_2_1 = (sigma_2_h1-sigma_2)/h ; 
D_sig_2_2 = (sigma_2_h2-sigma_2)/h ;
D_sig_2 = [ D_sig_2_1 ; D_sig_2_2 ]
% % 
D_sigma_2_form = - ( E_0'*n_0)
% 
sigma_3 =  + (f_0 + E_0*y_0)'*n_0 - f_max_0 ;
sigma_3_h1 =  + (f_0 + E_0*y_h_1)'*n_0 - f_max_0 ;
sigma_3_h2 =  + (f_0 + E_0*y_h_2)'*n_0 - f_max_0 ;
%
D_sig_3_1 = (sigma_3_h1-sigma_3)/h ;
D_sig_3_2 = (sigma_3_h2-sigma_3)/h ;
D_sig_3 = [ D_sig_3_1 ; D_sig_3_2 ]
D_sigma_3_form =  ( E_0'*n_0)
%
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Derivative Computations
% 1) D_V_y_fin_diff


% 2) D_V_y_con la formula


return
%%














% D_V, finite differences
fc_0 = 0*fc_0;
V_0 = V_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim )
y0 = [0 ; 0] ;
h=  sqrt(eps);
y_h1 = y0 ;
y_h1(1) = y_h1(1) +h ;
V_h1 = V_tot( fc_0+E_el*y_h1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ;
d_V_1 = (V_h1- V_0)/h ;
y_h2 = y0 ;
y_h2(2) = y_h2(2) +h ;
V_h2 = V_tot( fc_0+E_el*y_h2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim ) ;
d_V_2 = (V_h2- V_0)/h ;
D_V_y_finite = [d_V_1 ; d_V_2] 




D_V_function = D_V_y( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim , E, fc_0 )



return
[fc_opt, y,fval,exitflag,output, elapsed_time] = ...
    V_optimal_minunc_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect ,...
    cf_dim, E_el, fc_0 )


% 
return
%%
V_grad_y = V_grad_tot( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim, E_el, fc_0) 
% y0 = [0 ; 0]
% h=  sqrt(eps);
% y_h1 = y0 ;
% y_h1(1) = y_h1(1) +h ;
% V_h1 = V_tot( fc_0+E_el*y_h1, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );
% d_V_1 = (V_h1- V_0)/h 
% 
% 
% y_h2 = y0 ;
% y_h2(2) = y_h2(2) +h ;
% V_h2 = V_tot( fc_0+E_el*y_h2, normals_vect, mu_vect, f_min_vect, f_max_vect , cf_dim );
% d_V_2 = (V_h2- V_0)/h 

[fc_opt, y,fval,exitflag,output, elapsed_time] = ...
    V_optimal_minunc_grad( fc_0, normals_vect, mu_vect, f_min_vect, f_max_vect ,...
    cf_dim, E_el, fc_0 )


% toc
% fc_0
% fc_final = fc_0 + E_el*y
return
% 2) Computing V for a given coefficient vector y

% 3) Building the minimization function




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Quasi-Statics (rigido e elastico)
 % + numerical filters
 % calcolo derivate per i contact frame
 % calcolo derivate gravitazionali
% 1) FLMM

% 2) canonical form of the FLMM

% 3) controllable displacements (by the symbolic form vs by the cFLMM)

% 4) controllable contact forces (by the symbolic form vs by the cFLMM)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Contact Force Optimization 
% (by CVX vs by semi-custom optimization (V def, optim by matlab) vs complete-custom come Bicchi in paper)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Final Configuration Computation (si ingloba sopra?)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Final Plot, sequenza tra zero e 1, video
% utilizzare anche tool di kinematics


%%%%%%%%%%%%%%%
alpha_deg_0 = 10
alpha = deg2rad(alpha_deg_0) ;
%
mu_1 =   1 
mu_2 =   .3
%
mg_0 = - 500 ;
L1 = 0.5  ; % Link Lengths 
L2 = 0.5  ;
L3 = 0.5  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parametrized initial robot condiguration
beta_deg_0 = 60 ;
beta_0 = deg2rad(beta_deg_0) ;
D = sqrt( L2^2+ L3^2 -2*L2*L3*cos(beta_0)  ) ;
%
alpha_1 = acos( (D^2+L2^2-L3^2)/(2*L2*D) ) ;
alpha_1_deg = rad2deg(alpha_1) ;
%
beta_1 = pi/2-(alpha+alpha_1) ;
beta_2 = beta_0-beta_1 ;
h1 = L2*cos(beta_1)  ;
%
phi_1 = atan2(L2/3,L2)  ;
phi_2 = atan2(L2/3,L2)  ;
%
L2_aux = L2/(2*cos(phi_1)) ;
L3_aux = L3/(2*cos(phi_2)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%
f_min = 0 ;   % [N] minimum normal contact force allowable
f_max_leg = abs(mg_0) ; % [N] maximum normal contact force allowable 
%
h = sqrt(eps) ; 
%
kq = 1000 ;           % 
kc = 1000000000000  ; % 
% 
P1 = [ 0 0 0 ]'  ; % points belonging to the joint axis
P2 = [ 0 0 0 ]'  ;
P3 = [ 0 0 0 ]'  ;
P4 = [0 -L1 0]'  ;
P5 = [0 -L1 0]'  ;
P6 = P4    +   [-L2_aux*sin(beta_1-phi_1)     -L2_aux*cos(beta_1-phi_1)  0]'  ; % knee
P7 = P5    +   [ L3_aux*sin(beta_2+phi_1)     -L3_aux*cos(beta_2+phi_2)  0]'  ;
%
P = [P1, P2, P3, P4, P5, P6, P7 ] ;  % Collecting all the points
%
Z1 = [1, 0, 0]'   ;  % Joint axis versors
Z2 = [0, 1, 0]'   ;
Z3 = [0, 0, 1]'   ;
Z4 = [0, 0, 1]'   ;
Z5 = [0, 0, 1]'   ;
Z6 = [0, 0, 1]'   ;
Z7 = [0, 0, 1]'   ;
%
Z = [Z1, Z2, Z3, Z4, Z5, Z6, Z7 ]   ; % Collecting all the joint axis versors
%
% End-Effector Positions
EE_B  = [0 0 0]' ;  % Floating base
EE_G1 = [0 0 0]' ;  % Mass 
%
EE_C1 = P4    +   [-L2*sin(beta_1)     -L2*cos(beta_1)  0]'  ; %  First Contact
EE_C2 = P5    +   [ L3*sin(beta_2)     -L3*cos(beta_2)  0]'  ; %  Second Contact
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial Forces Computation
d1 = norm(EE_G1(1)-EE_C1(1)) ;
d2 = norm(EE_G1(1)-EE_C2(1)) ;
fc2_y_0 = mg_0*d1/(d1+d2); %  contact force from the robot to the environment
fc1_y_0 = - ((-mg_0)+fc2_y_0)
fc2_y_0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
joint_type_virtual = [1 1 0] ; % joint types of the Virtual Kinematic Chain % 1 = prismatic joint
joint_type_real =  [0 0] ;     % joint types of the Real Kinematic Chain    % 0 = revolute joint
joint_type = [joint_type_virtual, joint_type_real, joint_type_real] ;
%
% Contact Force Pre-load Definition 
g_0  = [0   mg_0 ]'  ;    %  
fc_0 = [0  fc1_y_0 , 0  fc2_y_0 ]'  ; % 
%
lambda = [fc_0;  -g_0] ; % exerted by the robot
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Spatial Jacobians of relevant frames
%
J_a_b_comp = spatial_WB_Jacobian([P1 P2 P3],[Z1 Z2 Z3], [1 1 0], [1:3] ) ; % spatial_WB_Jacobian( P, Z, joint_type, joint_chain )
J_a_g1_comp = J_a_b_comp   ;
J_a_c1_comp = spatial_WB_Jacobian([P1 P2 P3 P4 P6 ],[Z1 Z2 Z3 Z4 Z6], [1 1 0 0 0], [1:5] )  ;
J_a_c2_comp = spatial_WB_Jacobian([P1 P2 P3 P5 P7 ],[Z1 Z2 Z3 Z5 Z7], [1 1 0 0 0], [1:5] )  ;
%
% According dimensions to the problem
J_a_b_comp  = [J_a_b_comp, zeros(6,4)] ;
J_a_g1_comp = [J_a_g1_comp, zeros(6,4)] ;
J_a_c1_comp = [J_a_c1_comp(:,1:4), zeros(6,1),J_a_c1_comp(:,end),zeros(6,1)] ;
J_a_c2_comp = [J_a_c2_comp(:,1:3), zeros(6,1),J_a_c2_comp(:,4),zeros(6,1), J_a_c2_comp(:,5) ] ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
eye4 = eye(4); 
%
g_ab_0  = eye4 ;  % Initial Configuration of relevant frames
g_ag1_0 = eye4 ;
g_ac1_0 = eye4 ;
g_ac2_0 = eye4 ;
%
g_ab_0(1:3,4)  = EE_B   ;
g_ag1_0(1:3,4) = EE_G1  ;
g_ac1_0(1:3,4) = EE_C1  ;
g_ac2_0(1:3,4) = EE_C2  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Body Jacobians
J_b_b_comp   = ad2(ihomogeneous(g_ab_0))*J_a_b_comp ;
J_g1_g1_comp = ad2(ihomogeneous(g_ag1_0))*J_a_g1_comp  ;
%
J_c1_c1_comp = ad2(ihomogeneous(g_ac1_0))*J_a_c1_comp  ; 
J_c2_c2_comp = ad2(ihomogeneous(g_ac2_0))*J_a_c2_comp  ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
B_b_T  = [1 0 0 0 0 0; 0 1 0 0 0 0 ;  0 0 0 0 0 1 ] ;
B_g1_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ;
B_c1_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ; % hard finger
B_c2_T = [1 0 0 0 0 0; 0 1 0 0 0 0 ]  ;
%
J_b_b   = B_b_T*J_b_b_comp      ;
J_g1_g1 = B_g1_T*J_g1_g1_comp   ;
J_c1_c1 = B_c1_T*J_c1_c1_comp   ;
J_c2_c2 = B_c2_T*J_c2_c2_comp   ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
S_g_T = J_g1_g1(:,1:3) ;
S_g   = S_g_T' ;
J_g   = J_g1_g1(:,4:end) ;
%
J_c_n = [J_c1_c1; J_c2_c2 ]  ;
S_c_T = J_c_n(:,1:3)    ;
S_c = S_c_T' ;
J_c   = J_c_n(:,4:end)  ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Derivatives 
%
Q_c1 = D_J_ci(J_a_c1_comp, g_ac1_0 ,  B_c1_T,  fc_0(1:2)) ; 
Q_c2 = D_J_ci(J_a_c2_comp, g_ac2_0 ,  B_c2_T,  fc_0(3:4)) ; 
%
% Derivative of the Gravitational Term
% 
tau_g_0 = (B_g1_T*ad2(ihomogeneous(g_ag1_0))*J_a_g1_comp)'*lambda(5:6) ;
D_g = [] ;
for i= 1:3
%    
q_temp = [ 0 0 0 ];
q_temp(i) =   q_temp(i)+h ;
%
J_s_g1_temp = singleSerialJacobian(P(:,[1:3]),Z(:,[1:3]),q_temp,[1:3], [ 1 1 0]) ;
% 
J_s_g1_temp = [J_s_g1_temp, zeros(6,4)] ;
g_ag1_temp = forward_kinematics_NOSYM(P(:,[1:3]),Z(:,[1:3]),...
        g_ag1_0,q_temp([ 1:3 ]),[1 2])   ; 
g_ag1_temp(1:3,1:3) = eye(3) ;       % the frame does not rotate, can only translate
tau_g_i =(B_g1_T *  ad2( ihomogeneous(g_ag1_temp))*J_s_g1_temp  )'*lambda(5:6)  ; 
%tau_g_i = [ tau_g_i ; 0 ; 0]
    D_g = [D_g, (tau_g_i-tau_g_0)./h ] ;
end
D_g = [D_g, zeros(7,4)] ;
%
Q_tot =  Q_c1 + Q_c2 ;
%
U_s = Q_tot(1:3,1:3) ;
U_j = Q_tot(4:end,1:3) ;
Q_s = Q_tot(1:3,4:end);
Q_j = Q_tot(4:end,4:end) ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sizes
s_fc  = size(J_c,1) ; 
s_tau = size(J_c,2) ; 
s_u  = size(S_g,1) ; 
s_wb = size(S_g,1) ; 
s_q  = size(J_c,2) ; 
%
Kc = kc*eye(s_fc)  ; 
Kq = kq*eye(s_q) ; 
%
%
FLMM_1 = FLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
cFLMM_1 =  cFLMM_compact_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
FLMM_2 = FLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
cFLMM_2 = cFLMM_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
R_f = Rf_rigid(  J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
R_u = Ru_rigid( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc )  ;
R_tau = Rtau_rigid( J_c,  S_c, Q_j,  Q_s, U_j , U_s,  Kc ) ;
%
%
R_f_tot = R_f ;
R_f_filtr = round(R_f_tot*10^(-6))*10^6 ;
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Contact Force Optimization
%
% Normal Contact Vector
n_c1 = [0 -1]' ;
n_c2 = [cos( alpha-pi/2) ; sin(alpha-pi/2)  ]  ;
%
check_angle = rad2deg(acos(n_c1'*n_c2 ) ) ;
%
%
E = orth(R_f_filtr) %  
%
cvx_begin               % quiet % non-verbose method
cvx_precision  high     % low % quality solution option
%   cvx_solver sedumi   % alternative solver
variable y_cvx(size(E,2))
%
d_fc = E*y_cvx ;
fc_ott = fc_0+ d_fc ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
sigma_c1_frict = -( cos(atan(mu_1))*norm( fc_ott(1:2)) - fc_ott(1:2)'*n_c1 );
sigma_c2_frict =-( cos(atan(mu_2))*norm( fc_ott(3:4)) - fc_ott(3:4)'*n_c2 );
%
sigma_c1_min = -(f_min - ( fc_ott(1:2))'*n_c1 );
sigma_c2_min = -(f_min - ( fc_ott(3:4))'*n_c2 );
%
sigma_c1_max = -(-f_max_leg + ( fc_ott(1:2))'*n_c1 );
sigma_c2_max = -(-f_max_leg + ( fc_ott(3:4))'*n_c2 );
%
sigma_c1_ott = [pow_p(sigma_c1_frict,-2), pow_p(sigma_c1_min,-2), pow_p(sigma_c1_max,-2)] ;
sigma_c2_ott = [pow_p(sigma_c2_frict,-2), pow_p(sigma_c2_min,-2), pow_p(sigma_c2_max,-2)] ;
% %
sigma_fc_tot = [ sigma_c1_ott, sigma_c2_ott  ]' ;
% 
V_fc_tot = sum((sigma_fc_tot)) ; %norm(sigma_c1_min)%%sum(pow_p( sigma_fc_tot(1), -2 )) ;%
%
%
minimize( V_fc_tot )   
%        subject to
%         sigma_fc_tot >= 0 ;
cvx_end
%
% Recomputation of the last step results
%
d_fc = E*y_cvx ;
fc_ott = fc_0+ d_fc ;
%
sigma_c1_frict = cos(atan(mu_1))*norm( fc_ott(1:2)) - fc_ott(1:2)'*n_c1 ;
sigma_c2_frict = cos(atan(mu_2))*norm( fc_ott(3:4)) - fc_ott(3:4)'*n_c2 ;
%
sigma_c1_min = f_min - ( fc_ott(1:2))'*n_c1 ;
sigma_c2_min = f_min - ( fc_ott(3:4))'*n_c2 ;
%
sigma_c1_max = -f_max_leg + ( fc_ott(1:2))'*n_c1 ;
snapnow;
sigma_c2_max = -f_max_leg + ( fc_ott(3:4))'*n_c2 ;
%
sigma_c1_ott = [sigma_c1_frict, sigma_c1_min, sigma_c1_max] ;
sigma_c2_ott = [sigma_c2_frict, sigma_c2_min, sigma_c2_max] ;
%
sigma_fc_tot = [ sigma_c1_ott, sigma_c2_ott  ]' ;
% 
V_fc_tot = sum(exp(sigma_fc_tot)) ;
%
d_fc_tot_wb = E*y_cvx ;
fc_tot_wb = fc_0 + d_fc_tot_wb  % 
%
d_q = pinv(-R_f_tot)*d_fc_tot_wb
d_u = -R_u*d_q
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Section
P_Plot = [EE_G1, P4 , P5, P6, P7]  ;
%
Z_plot = [ Z3, Z4 Z5 Z6 Z7]    ;
EE_plot = [ EE_C1 , EE_C2  ]  ;
%
close all
figure
grid on
axis equal
axis( [ -.35 .55  -1 .1 ] ) % 
%  
plot_Robot(P_Plot, Z_plot, EE_plot, [0 0 0 0 0], [0 0 0 0 0], [1 2 4], [1 3 5]);
plot_Object(homogeneous( eye(3) , EE_G1 ) ,.05,0,'Sphere');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Plot of the final configuration
%
EE_G1_final =  forward_kinematics_NOSYM( [P1 P2 P3] , [Z1 Z2 Z3],  homogeneous( eye(3) , EE_G1 )  , d_u , [1 2] ) ;
EE_G1_final = EE_G1_final(1:3,4)  ;
% 
P3_final =  forward_kinematics_NOSYM( [P1 P2 ] , [Z1 Z2 ],  homogeneous( eye(3) , P3 )  , d_u(1:2) , [1 2] ) ;
%
P4_final = forward_kinematics_NOSYM( [P1 P2 P3] ,...
    [Z1 Z2 Z3 ],  homogeneous( eye(3) , P4 )  , [ d_u; ] , [ 1 2 ] ) ;
P4_final = P4_final(1:3,4)  ;
%
P5_final  = forward_kinematics_NOSYM( [P1 P2 P3 ] ,...
    [Z1 Z2 Z3 ],  homogeneous( eye(3) , P5 )  , [ d_u; ] , [ 1 2 ] );
P5_final = P5_final(1:3,4)  ;
%
P6_final  = forward_kinematics_NOSYM( [P1 P2 P3 P4 ] ,...
    [Z1 Z2 Z3 Z4 ],  homogeneous( eye(3) , P6 )  , [ d_u; d_q(1) ] , [ 1 2 ] );
P6_final = P6_final(1:3,4)  ;
%
P7_final  = forward_kinematics_NOSYM( [P1 P2 P3 P5 ] ,...
    [Z1 Z2 Z3 Z5 ],  homogeneous( eye(3) , P7 )  , [ d_u; d_q(2) ] , [ 1 2 ] ) ;
P7_final = P7_final(1:3,4)  ;
%  
% %
d_C = S_c_T*d_u+J_c*d_q ;
EE_C1_final = [ EE_C1(1:2) + d_C(1:2);0 ] ;
EE_C2_final = [ EE_C2(1:2) + d_C(3:4);0 ] ;
EE_plot_final = [ EE_C1_final , EE_C2_final  ]  ;
%
P_Plot_final = [EE_G1_final,  P4_final , P5_final, P6_final, P7_final]   ;
%
figure
grid on
axis equal
axis( [ -.35 .55  -1 .1 ] )
plot_Robot(P_Plot_final ,Z_plot, EE_plot_final,[0 0 0 0 0], [0 0 0 0 0], [1 2 4], [1 3 5]);
plot_Object( homogeneous( eye(3) , EE_G1_final ) ,.05,0,'Sphere');
%





















