%    spatial_WB_Jacobian_q
%   
%    Computes the configuration dependent spatial Jacobian
%
%    The function calculates the Whole-Body spatial Jacobian for the given
%    input configuration, using twist notation.
%
%    Usage: J = spatial_WB_Jacobian_q(J,q)
%
%    Inputs: [...]
%
%    Returns:
%    J(q) = the spatial Whole-Body Jacobian
%

%

function J_q = spatial_WB_Jacobian_q(J_0,q)
[r,c] = size(J_0,2) ;
s_q = length(q) ;
if(s_q~=c)
    error('The dimension of J_0 and q do not agree')
end
J_temp = zeros(r,c) ;
J_temp(:,1) = J_0(:,1) ;
for i = 2 : c
    %clear Ad_temp ;
    Ad_temp = eye(6) ;
    for j=1:i-1
        Ad_temp = Ad_temp* ad2( twistexp2( J_0(:,j)*q(j) )  )
    end
    J_temp(:,i) = Ad_temp* J_0(:,i) ;
   
end

J_q = J_temp ;
    
    

end
return
% function J = spatial_WB_Jacobian(P,Z,joints,varargin)
% %
% if all(size(joints)==[1,1])
%    joints = joints*ones(1,size(P,2));
% end
% %
% q0 = zeros(size(joints)); %q0 = joints ;
% q = size(P,2); % joints number
% if(size(q0,1) == 1 && size(q0,2) == 1)
%     q0 = sign(q0)^2*ones(1,q);
% else
%     if(size(q0,1) == q || size(q0,2) == q)
%     else
%         error('q0 must be a [1 x num_joints] vector, or a single scalar');
%     end
% end
%         
% if(is_row(Z) || is_column(Z)) % the second input is only a vector, means  
%     if(is_row(Z))             % that all the joints have the same joint axis
%         Z = Z';
%     end
%     z = Z;
%     Z = [];
%     for i = 1 : q
%         Z = [Z z];
%     end
% end
%         
% if(size(P) ~= size(Z))
%     error('P and Z must be the same size');
% end
% 
% n_arg = nargin;
% if(iscell(varargin{nargin-3}))    % if the last input is a cell array,
%     vect = varargin{nargin-3} ;    % means that is the prismatic joints configuration
%     n_arg = n_arg-1;
% end
% 
% %joints = zeros(1,q);
% 
% if(exist('vect','var'))
%     for i = 1 : size(vect,2)
%     joints(vect{i}) = 1;    % for each joint, 1 represents the prismatic joint
%     end
% end
% 
% conf = [];
% J = [];
% 
% for i = 1 : n_arg-3 
%   
%     act_conf = varargin{i};
%     s = size(act_conf,2);
%     act_q0 = [];
%     for k = 1 : s
%         act_q0 = [act_q0 q0(act_conf(k))];
%     end
%     
%     % for each serial configuration, calculates the Jacobian
%     J_chain_i = singleSerialJacobian(P,Z,act_q0,act_conf,joints);
% 
%     % looks in the configuration to find how to merge the jacobians
%     c = intersect(conf,act_conf);   
%     if(isempty(c)) % case of two indipendent serial chains
%         J = blkdiag(J,J_chain_i);
%         conf = [conf act_conf];
%     else  % if the global configuration has joints in common to the actual
%         k = size(J,2);
%         l = size(J,1);
% %         J_inf = sym(zeros(6,k+s-size(c,2)));
%         J_inf = zeros(6,k+s-size(c,2));
%         for m = 1 : s
%             J_inf(:,act_conf(m)) = J_chain_i(:,m);
%         end
%         
%         J_vert = zeros(l,s-size(c,2));
%         J = [J J_vert;
%              J_inf];
%         
%         conf = [conf act_conf];
%         conf = unique(conf);    % remove duplicates
%     end
% end
% 
% end
